/*
###
### Draw marker at round numbers or user defined price levels
###
### User		Date 		Description
### ------	-------- 	-------------
### Gaston	June 2011	Created by monpere
###
*/
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Text.RegularExpressions;
#endregion

namespace NinjaTrader.Indicator
{
    [Description("Draw marker at round numbers or user defined price levels")]
    public class RoundNumbers : Indicator
    {
        #region Variables
		
		string tikFmt="";	
		string usrFmt="";	
		string numFmt="";	
		int    numLineLen   = 5;				
		Color  numLineColor = Color.Yellow;	
		
		double d=0, lastMin=0, lastMax=0, lastScreenBar=0, lastBar=0;
		
	    Pen linePen = new Pen(Color.Yellow,  2);
		
        #endregion

        protected override void Initialize() {
			AutoScale 			= false;
            Overlay				= true;
			CalculateOnBarClose = false;
			BarsRequired = 1;
        }

        protected override void OnBarUpdate() {
			if ( CurrentBar == 1 ) {
				tikFmt = Regex.Replace(TickSize.ToString(),"([\\d])","0");
				numFmt = "00$";												
				if      ( tikFmt.EndsWith(".0") )     numFmt = "0.0$";		
				else if ( tikFmt.EndsWith(".00000") ) numFmt = "00000$";	
				else if ( tikFmt.EndsWith(".000000") )numFmt = "000000$";	
				else if ( tikFmt.EndsWith(".0000000"))numFmt = "000000$";	

				if ( !String.IsNullOrEmpty(usrFmt) ) {
					numFmt = usrFmt;
					numFmt = Regex.Replace(numFmt,","," ");
					numFmt = Regex.Replace(numFmt,"\\s+"," ");
					numFmt = numFmt.Trim();
					numFmt = Regex.Replace(numFmt," ","$|") + "$";
				}
			}
		}
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max) {
			try {
				if ( min != lastMin || max != lastMax || ChartControl.LastBarPainted != lastScreenBar) {
					int x = CurrentBar - ChartControl.LastBarPainted - 1; if (x<0 || x>CurrentBar) x=0;
					for (d=min; d < max; ) {
						if ( d > 0 && Regex.Match(d.ToString(tikFmt),numFmt).Success ) {
							DrawLine("nL"+d, false, x, d, Math.Min(x+numLineLen,CurrentBar), d, numLineColor, DashStyle.Solid, 2);
						}
						d = Instrument.MasterInstrument.Round2TickSize(d+TickSize);
					}
				}
			}
			catch (Exception ex) {
	    	    Print("(ROUND_NUMBERS) Exception: " + ex.Message);
			}
			
			lastMin = min; lastMax = max;
			lastScreenBar = ChartControl.LastBarPainted;
		}		

        #region Properties
		

        [Description("Mark numbers ending in these digits. Space or comma delimited list Ex: 0, 00, .25, 0.0, .0010")]
		[Gui.Design.DisplayNameAttribute("1. Numbers Ending in:")]
        public string UsrFmt
        {
            get { return usrFmt; }
			set { usrFmt = value; }
        }
		
        [Description("Line Length (in bars)")]
		[Gui.Design.DisplayNameAttribute("2. Line Length")]
        public int NumLineLen
        {
            get { return numLineLen; }
			set { numLineLen = value; }
        }
		
        [Description("Number Line Color")]
        [Gui.Design.DisplayNameAttribute("3. Line Color")]
        public Color NumLineColor
        {
            get { return numLineColor; }
            set { numLineColor = value; }
        }
        [Browsable(false)]
        public string NumLineColorSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(numLineColor); }
            set { numLineColor = Gui.Design.SerializableColor.FromString(value); }
        }		
				
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RoundNumbers[] cacheRoundNumbers = null;

        private static RoundNumbers checkRoundNumbers = new RoundNumbers();

        /// <summary>
        /// Draw marker at round numbers or user defined price levels
        /// </summary>
        /// <returns></returns>
        public RoundNumbers RoundNumbers()
        {
            return RoundNumbers(Input);
        }

        /// <summary>
        /// Draw marker at round numbers or user defined price levels
        /// </summary>
        /// <returns></returns>
        public RoundNumbers RoundNumbers(Data.IDataSeries input)
        {
            if (cacheRoundNumbers != null)
                for (int idx = 0; idx < cacheRoundNumbers.Length; idx++)
                    if (cacheRoundNumbers[idx].EqualsInput(input))
                        return cacheRoundNumbers[idx];

            lock (checkRoundNumbers)
            {
                if (cacheRoundNumbers != null)
                    for (int idx = 0; idx < cacheRoundNumbers.Length; idx++)
                        if (cacheRoundNumbers[idx].EqualsInput(input))
                            return cacheRoundNumbers[idx];

                RoundNumbers indicator = new RoundNumbers();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                RoundNumbers[] tmp = new RoundNumbers[cacheRoundNumbers == null ? 1 : cacheRoundNumbers.Length + 1];
                if (cacheRoundNumbers != null)
                    cacheRoundNumbers.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheRoundNumbers = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Draw marker at round numbers or user defined price levels
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RoundNumbers RoundNumbers()
        {
            return _indicator.RoundNumbers(Input);
        }

        /// <summary>
        /// Draw marker at round numbers or user defined price levels
        /// </summary>
        /// <returns></returns>
        public Indicator.RoundNumbers RoundNumbers(Data.IDataSeries input)
        {
            return _indicator.RoundNumbers(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Draw marker at round numbers or user defined price levels
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RoundNumbers RoundNumbers()
        {
            return _indicator.RoundNumbers(Input);
        }

        /// <summary>
        /// Draw marker at round numbers or user defined price levels
        /// </summary>
        /// <returns></returns>
        public Indicator.RoundNumbers RoundNumbers(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RoundNumbers(input);
        }
    }
}
#endregion
