#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Design;
#endregion

#region Public Enumerations
public enum MAPackageMATypeV13: uint {SMA, EMA, HMA, TMA, TEMA, VWMA, WMA, ZLEMA, LLMA, T3, LinReg, JMA,
										ButterWorthFilter, GaussianFilter, SuperSmootherFilter};
public enum MAPackageMAPriceTypesV13: uint {Open, Close, High, Low, Median, Typical, Weighted};
public enum MAPackageDashStyleV13: uint {Solid, Dash, DashDot, DashDotDot, Dot};
public enum MAPackagePlotStyleV13: uint {Bar, Block, Cross, Dot, Hash, HLine, Line, Square, TriangleDown, TriangleLeft, TriangleRight, TriangleUp};
#endregion

#region Versions
/// Created by Erezw, 26.03.2012, Version 1.2 Release.
/// Updated by Erezw, 29.06.2012, Version 1.3 Add LLMA,T3 MA, dynamic properties.
#endregion

namespace NinjaTrader.Indicator
{
    [Description("MAPackage V1.3")]
    public class MAPackage : Indicator, ICustomTypeDescriptor
    {
		#region Private Enumerations
		private enum MADirectionTypes: uint {Rising, Falling};
		#endregion
		
		#region Variables
		#region Indicators Parameters
		private bool _UseMA1 = true;
		private bool _UseMA2 = true;
		private bool _UseMA3 = true;
		private bool _UseMA4 = true;
		private bool _UseMA5 = true;
		private bool _UseMA6 = true;
		private bool _UseMA7 = false;
		private DataSeries _MA1Value;
		private DataSeries _MA2Value;
		private DataSeries _MA3Value;
		private DataSeries _MA4Value;
		private DataSeries _MA5Value;
		private DataSeries _MA6Value;
		private DataSeries _MA7Value;
		private int _MA1Period = 1;
		private int _MA2Period = 3;
		private int _MA3Period = 13;
		private int _MA4Period = 20;
		private int _MA5Period = 50;
		private int _MA6Period = 100;
		private int _MA7Period = 200;
		private int _JMAPhase1 = 0;
		private int _JMAPhase2 = 0;
		private int _JMAPhase3 = 0;
		private int _JMAPhase4 = 0;
		private int _JMAPhase5 = 0;
		private int _JMAPhase6 = 0;
		private int _JMAPhase7 = 0;
		private int _T3tCount1 = 3;
		private int _T3tCount2 = 3;
		private int _T3tCount3 = 3;
		private int _T3tCount4 = 3;
		private int _T3tCount5 = 3;
		private int _T3tCount6 = 3;
		private int _T3tCount7 = 3;
		private double _T3vFactor1 = 0.7;
		private double _T3vFactor2 = 0.7;
		private double _T3vFactor3 = 0.7;
		private double _T3vFactor4 = 0.7;
		private double _T3vFactor5 = 0.7;
		private double _T3vFactor6 = 0.7;
		private double _T3vFactor7 = 0.7;
		private int _ButterWorthFilterPoles1 = 3;
		private int _ButterWorthFilterPoles2 = 3;
		private int _ButterWorthFilterPoles3 = 3;
		private int _ButterWorthFilterPoles4 = 3;
		private int _ButterWorthFilterPoles5 = 3;
		private int _ButterWorthFilterPoles6 = 3;
		private int _ButterWorthFilterPoles7 = 3;
		private int _GaussianFilterPoles1 = 3;
		private int _GaussianFilterPoles2 = 3;
		private int _GaussianFilterPoles3 = 3;
		private int _GaussianFilterPoles4 = 3;
		private int _GaussianFilterPoles5 = 3;
		private int _GaussianFilterPoles6 = 3;
		private int _GaussianFilterPoles7 = 3;
		private int _SuperSmootherFilterPoles1 = 3;
		private int _SuperSmootherFilterPoles2 = 3;
		private int _SuperSmootherFilterPoles3 = 3;
		private int _SuperSmootherFilterPoles4 = 3;
		private int _SuperSmootherFilterPoles5 = 3;
		private int _SuperSmootherFilterPoles6 = 3;
		private int _SuperSmootherFilterPoles7 = 3;
		private MADirectionTypes MA1Direction = MADirectionTypes.Rising;
		private MADirectionTypes MA2Direction = MADirectionTypes.Rising;
		private MADirectionTypes MA3Direction = MADirectionTypes.Rising;
		private MADirectionTypes MA4Direction = MADirectionTypes.Rising;
		private MADirectionTypes MA5Direction = MADirectionTypes.Rising;
		private MADirectionTypes MA6Direction = MADirectionTypes.Rising;
		private MADirectionTypes MA7Direction = MADirectionTypes.Rising;
		private MAPackageMATypeV13 _MA1Type = MAPackageMATypeV13.SMA;
		private MAPackageMATypeV13 _MA2Type = MAPackageMATypeV13.SMA;
		private MAPackageMATypeV13 _MA3Type = MAPackageMATypeV13.EMA;
		private MAPackageMATypeV13 _MA4Type = MAPackageMATypeV13.SMA;
		private MAPackageMATypeV13 _MA5Type = MAPackageMATypeV13.SMA;
		private MAPackageMATypeV13 _MA6Type = MAPackageMATypeV13.SMA;
		private MAPackageMATypeV13 _MA7Type = MAPackageMATypeV13.SMA;
		private MAPackageMAPriceTypesV13 _MA1PriceType = MAPackageMAPriceTypesV13.Typical;
		private MAPackageMAPriceTypesV13 _MA2PriceType = MAPackageMAPriceTypesV13.Typical;
		private MAPackageMAPriceTypesV13 _MA3PriceType = MAPackageMAPriceTypesV13.Close;
		private MAPackageMAPriceTypesV13 _MA4PriceType = MAPackageMAPriceTypesV13.Close;
		private MAPackageMAPriceTypesV13 _MA5PriceType = MAPackageMAPriceTypesV13.Close;
		private MAPackageMAPriceTypesV13 _MA6PriceType = MAPackageMAPriceTypesV13.Close;
		private MAPackageMAPriceTypesV13 _MA7PriceType = MAPackageMAPriceTypesV13.Close;
		#endregion
		
		#region  Plot
		private Color _MA1RisingColor = Color.Blue;
		private Color _MA2RisingColor = Color.Sienna;
		private Color _MA3RisingColor = Color.LimeGreen;
		private Color _MA4RisingColor = Color.Magenta;
		private Color _MA5RisingColor = Color.Blue;
		private Color _MA6RisingColor = Color.DimGray;
		private Color _MA7RisingColor = Color.MidnightBlue;
		private int _MA1RisingColorOpacity = 255;
		private int _MA2RisingColorOpacity = 255;
		private int _MA3RisingColorOpacity = 255;
		private int _MA4RisingColorOpacity = 255;
		private int _MA5RisingColorOpacity = 255;
		private int _MA6RisingColorOpacity = 255;
		private int _MA7RisingColorOpacity = 255;
		private Color _MA1FallingColor = Color.MidnightBlue;
		private Color _MA2FallingColor = Color.Maroon;
		private Color _MA3FallingColor = Color.DarkGreen;
		private Color _MA4FallingColor = Color.DarkMagenta;
		private Color _MA5FallingColor = Color.MidnightBlue;
		private Color _MA6FallingColor = Color.Black;
		private Color _MA7FallingColor = Color.MidnightBlue;
		private int _MA1FallingColorOpacity = 255;
		private int _MA2FallingColorOpacity = 255;
		private int _MA3FallingColorOpacity = 255;
		private int _MA4FallingColorOpacity = 255;
		private int _MA5FallingColorOpacity = 255;
		private int _MA6FallingColorOpacity = 255;
		private int _MA7FallingColorOpacity = 255;
		private int _MA1LineWidth = 3;
		private int _MA2LineWidth = 3;
		private int _MA3LineWidth = 3;
		private int _MA4LineWidth = 4;
		private int _MA5LineWidth = 4;
		private int _MA6LineWidth = 3;
		private int _MA7LineWidth = 1;
		private MAPackageDashStyleV13 _MA1LineDashStyle = MAPackageDashStyleV13.Dash;
		private MAPackageDashStyleV13 _MA2LineDashStyle = MAPackageDashStyleV13.Dash;
		private MAPackageDashStyleV13 _MA3LineDashStyle = MAPackageDashStyleV13.Solid;
		private MAPackageDashStyleV13 _MA4LineDashStyle = MAPackageDashStyleV13.Solid;
		private MAPackageDashStyleV13 _MA5LineDashStyle = MAPackageDashStyleV13.Dot;
		private MAPackageDashStyleV13 _MA6LineDashStyle = MAPackageDashStyleV13.Solid;
		private MAPackageDashStyleV13 _MA7LineDashStyle = MAPackageDashStyleV13.Dash;
		private MAPackagePlotStyleV13 _MA1LinePlotStyle = MAPackagePlotStyleV13.Line;
		private MAPackagePlotStyleV13 _MA2LinePlotStyle = MAPackagePlotStyleV13.Line;
		private MAPackagePlotStyleV13 _MA3LinePlotStyle = MAPackagePlotStyleV13.Line;
		private MAPackagePlotStyleV13 _MA4LinePlotStyle = MAPackagePlotStyleV13.Line;
		private MAPackagePlotStyleV13 _MA5LinePlotStyle = MAPackagePlotStyleV13.Line;
		private MAPackagePlotStyleV13 _MA6LinePlotStyle = MAPackagePlotStyleV13.Line;
		private MAPackagePlotStyleV13 _MA7LinePlotStyle = MAPackagePlotStyleV13.Line;
		private DashStyle _DefaultDashStyle = DashStyle.Solid;
		private PlotStyle _DefaultPlotStyle = PlotStyle.Line;
		#endregion
		#endregion
		
		#region Initialize
        protected override void Initialize()
        {
			Add(new Plot(Color.Empty, "MA1")); Plots[0].Pen.Width = _MA1LineWidth; Plots[0].PlotStyle = _DefaultPlotStyle; Plots[0].Pen.DashStyle = _DefaultDashStyle;
			Plots[0].Pen.Color = Color.FromArgb(_MA1RisingColorOpacity,_MA1RisingColor);
			
			Add(new Plot(Color.Empty, "MA2")); Plots[1].Pen.Width = _MA2LineWidth; Plots[1].PlotStyle = _DefaultPlotStyle; Plots[1].Pen.DashStyle = _DefaultDashStyle;
			Plots[1].Pen.Color = Color.FromArgb(_MA2RisingColorOpacity,_MA2RisingColor);
			
			Add(new Plot(Color.Empty, "MA3")); Plots[2].Pen.Width = _MA3LineWidth; Plots[2].PlotStyle = _DefaultPlotStyle; Plots[2].Pen.DashStyle = _DefaultDashStyle;
			Plots[2].Pen.Color = Color.FromArgb(_MA3RisingColorOpacity,_MA3RisingColor);
			
			Add(new Plot(Color.Empty, "MA4")); Plots[3].Pen.Width = _MA4LineWidth; Plots[3].PlotStyle = _DefaultPlotStyle; Plots[3].Pen.DashStyle = _DefaultDashStyle;
			Plots[3].Pen.Color = Color.FromArgb(_MA4RisingColorOpacity,_MA4RisingColor);
			
			Add(new Plot(Color.Empty, "MA5")); Plots[4].Pen.Width = _MA5LineWidth; Plots[4].PlotStyle = _DefaultPlotStyle; Plots[4].Pen.DashStyle = _DefaultDashStyle;
			Plots[4].Pen.Color = Color.FromArgb(_MA5RisingColorOpacity,_MA5RisingColor);
			
			Add(new Plot(Color.Empty, "MA6")); Plots[5].Pen.Width = _MA6LineWidth; Plots[5].PlotStyle = _DefaultPlotStyle; Plots[5].Pen.DashStyle = _DefaultDashStyle;
			Plots[5].Pen.Color = Color.FromArgb(_MA6RisingColorOpacity,_MA6RisingColor);
			
			Add(new Plot(Color.Empty, "MA7")); Plots[6].Pen.Width = _MA7LineWidth; Plots[6].PlotStyle = _DefaultPlotStyle; Plots[6].Pen.DashStyle = _DefaultDashStyle;
			Plots[6].Pen.Color = Color.FromArgb(_MA7RisingColorOpacity,_MA7RisingColor);
			
			_MA1Value = new DataSeries(this);
			_MA2Value = new DataSeries(this);
			_MA3Value = new DataSeries(this);
			_MA4Value = new DataSeries(this);
			_MA5Value = new DataSeries(this);
			_MA6Value = new DataSeries(this);
			_MA7Value = new DataSeries(this);
			
            Overlay = true;
			AutoScale = true;
			PriceTypeSupported = false;
			DrawOnPricePanel = true;
			PlotsConfigurable = false;
			CalculateOnBarClose = false;			
        }
		#endregion
		
		#region OnStartUp
		protected override void OnStartUp() 
		{
			#region MA1 Line Dash Style
			switch (_MA1LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[0].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[0].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[0].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[0].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[0].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion									
			
			#region MA2 Line Dash Style
			switch (_MA2LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[1].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[1].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[1].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[1].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[1].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA3 Line Dash Style
			switch (_MA3LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[2].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[2].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[2].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[2].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[2].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion									
			
			#region MA4 Line Dash Style
			switch (_MA4LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[3].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[3].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[3].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[3].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[3].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion									
			
			#region MA5 Line Dash Style
			switch (_MA5LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[4].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[4].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[4].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[4].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[4].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion									
			
			#region MA6 Line Dash Style
			switch (_MA6LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[5].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[5].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[5].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[5].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[5].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion									
			
			#region MA7 Line Dash Style
			switch (_MA7LineDashStyle)
			{
				case MAPackageDashStyleV13.Dash:
				{
					Plots[6].Pen.DashStyle = DashStyle.Dash;
					break;
				}
				case MAPackageDashStyleV13.DashDot:
				{
					Plots[6].Pen.DashStyle = DashStyle.DashDot;
					break;
				}
				case MAPackageDashStyleV13.DashDotDot:
				{
					Plots[6].Pen.DashStyle = DashStyle.DashDotDot;
					break;
				}
				case MAPackageDashStyleV13.Dot:
				{
					Plots[6].Pen.DashStyle = DashStyle.Dot;
					break;
				}
				case MAPackageDashStyleV13.Solid:
				{
					Plots[6].Pen.DashStyle = DashStyle.Solid;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion									
			
			#region MA1 Line Plot Style
			switch (_MA1LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[0].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[0].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[0].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[0].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[0].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[0].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[0].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[0].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[0].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[0].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[0].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[0].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA2 Line Plot Style
			switch (_MA2LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[1].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[1].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[1].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[1].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[1].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[1].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[1].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[1].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[1].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[1].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[1].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[1].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA3 Line Plot Style
			switch (_MA3LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[2].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[2].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[2].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[2].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[2].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[2].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[2].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[2].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[2].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[2].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[2].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[2].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA4 Line Plot Style
			switch (_MA4LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[3].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[3].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[3].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[3].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[3].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[3].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[3].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[3].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[3].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[3].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[3].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[3].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA5 Line Plot Style
			switch (_MA5LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[4].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[4].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[4].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[4].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[4].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[4].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[4].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[4].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[4].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[4].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[4].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[4].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA6 Line Plot Style
			switch (_MA6LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[5].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[5].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[5].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[5].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[5].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[5].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[5].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[5].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[5].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[5].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[5].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[5].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA7 Line Plot Style
			switch (_MA7LinePlotStyle)
			{
				case MAPackagePlotStyleV13.Bar:
				{
					Plots[6].PlotStyle = PlotStyle.Bar;
					break;
				}
				case MAPackagePlotStyleV13.Block:
				{
					Plots[6].PlotStyle = PlotStyle.Block;
					break;
				}
				case MAPackagePlotStyleV13.Cross:
				{
					Plots[6].PlotStyle = PlotStyle.Cross;
					break;
				}
				case MAPackagePlotStyleV13.Dot:
				{
					Plots[6].PlotStyle = PlotStyle.Dot;
					break;
				}
				case MAPackagePlotStyleV13.Hash:
				{
					Plots[6].PlotStyle = PlotStyle.Hash;
					break;
				}
				case MAPackagePlotStyleV13.HLine:
				{
					Plots[6].PlotStyle = PlotStyle.HLine;
					break;
				}
				case MAPackagePlotStyleV13.Line:
				{
					Plots[6].PlotStyle = PlotStyle.Line;
					break;
				}
				case MAPackagePlotStyleV13.Square:
				{
					Plots[6].PlotStyle = PlotStyle.Square;
					break;
				}
				case MAPackagePlotStyleV13.TriangleDown:
				{
					Plots[6].PlotStyle = PlotStyle.TriangleDown;
					break;
				}
				case MAPackagePlotStyleV13.TriangleLeft:
				{
					Plots[6].PlotStyle = PlotStyle.TriangleLeft;
					break;
				}
				case MAPackagePlotStyleV13.TriangleRight:
				{
					Plots[6].PlotStyle = PlotStyle.TriangleRight;
					break;
				}
				case MAPackagePlotStyleV13.TriangleUp:
				{
					Plots[6].PlotStyle = PlotStyle.TriangleUp;
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA1 Line Width
			Plots[0].Pen.Width = _MA1LineWidth;
			#endregion
			
			#region MA2 Line Width
			Plots[1].Pen.Width = _MA2LineWidth;
			#endregion
			
			#region MA3 Line Width
			Plots[2].Pen.Width = _MA3LineWidth;
			#endregion
			
			#region MA4 Line Width
			Plots[3].Pen.Width = _MA4LineWidth;
			#endregion
			
			#region MA5 Line Width
			Plots[4].Pen.Width = _MA5LineWidth;
			#endregion
			
			#region MA6 Line Width
			Plots[5].Pen.Width = _MA6LineWidth;
			#endregion
			
			#region MA7 Line Width
			Plots[6].Pen.Width = _MA7LineWidth;
			#endregion
			
			#region MA1 Type
			switch (_MA1Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = SMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = SMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = SMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = SMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = SMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = SMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = SMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = EMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = EMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = EMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = EMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = EMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = EMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = EMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = HMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = HMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = HMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = HMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = HMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = HMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = HMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = TMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = TMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = TMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = TMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = TMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = TMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = TMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = TEMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = TEMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = TEMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = TEMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = TEMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = TEMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = TEMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = T3(Open, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = T3(Close, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = T3(High, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = T3(Low, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = T3(Median, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = T3(Typical, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = T3(Weighted, _MA1Period, _T3tCount1, _T3vFactor1).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = VWMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = VWMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = VWMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = VWMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = VWMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = VWMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = VWMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = WMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = WMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = WMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = WMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = WMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = WMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = WMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = ZLEMA(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = ZLEMA(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = ZLEMA(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = ZLEMA(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = ZLEMA(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = ZLEMA(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = ZLEMA(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = LLMA(Open, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = LLMA(Close, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = LLMA(High, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = LLMA(Low, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = LLMA(Median, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = LLMA(Typical, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = LLMA(Weighted, _MA1Period, _JMAPhase1).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = LinReg(Open, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = LinReg(Close, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = LinReg(High, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = LinReg(Low, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = LinReg(Median, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = LinReg(Typical, _MA1Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = LinReg(Weighted, _MA1Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = JMA(Open, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = JMA(Close, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = JMA(High, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = JMA(Low, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = JMA(Median, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = JMA(Typical, _MA1Period, _JMAPhase1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = JMA(Weighted, _MA1Period, _JMAPhase1).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = anaButterworthFilter(Open, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = anaButterworthFilter(Close, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = anaButterworthFilter(High, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = anaButterworthFilter(Low, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = anaButterworthFilter(Median, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = anaButterworthFilter(Typical, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = anaButterworthFilter(Weighted, _MA1Period, _ButterWorthFilterPoles1).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = anaGaussianFilter(Open, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = anaGaussianFilter(Close, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = anaGaussianFilter(High, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = anaGaussianFilter(Low, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = anaGaussianFilter(Median, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = anaGaussianFilter(Typical, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = anaGaussianFilter(Weighted, _MA1Period, _GaussianFilterPoles1).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA1PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA1Value = anaSuperSmootherFilter(Open, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA1Value = anaSuperSmootherFilter(Close, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA1Value = anaSuperSmootherFilter(High, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA1Value = anaSuperSmootherFilter(Low, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA1Value = anaSuperSmootherFilter(Median, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA1Value = anaSuperSmootherFilter(Typical, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA1Value = anaSuperSmootherFilter(Weighted, _MA1Period, _SuperSmootherFilterPoles1).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA2 Type
			switch (_MA2Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = SMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = SMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = SMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = SMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = SMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = SMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = SMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = EMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = EMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = EMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = EMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = EMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = EMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = EMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = HMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = HMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = HMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = HMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = HMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = HMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = HMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = TMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = TMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = TMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = TMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = TMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = TMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = TMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = TEMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = TEMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = TEMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = TEMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = TEMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = TEMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = TEMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = T3(Open, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = T3(Close, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = T3(High, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = T3(Low, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = T3(Median, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = T3(Typical, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = T3(Weighted, _MA2Period, _T3tCount2, _T3vFactor2).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = VWMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = VWMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = VWMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = VWMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = VWMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = VWMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = VWMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = WMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = WMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = WMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = WMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = WMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = WMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = WMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = ZLEMA(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = ZLEMA(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = ZLEMA(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = ZLEMA(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = ZLEMA(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = ZLEMA(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = ZLEMA(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = LLMA(Open, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = LLMA(Close, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = LLMA(High, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = LLMA(Low, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = LLMA(Median, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = LLMA(Typical, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = LLMA(Weighted, _MA2Period, _JMAPhase2).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = LinReg(Open, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = LinReg(Close, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = LinReg(High, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = LinReg(Low, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = LinReg(Median, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = LinReg(Typical, _MA2Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = LinReg(Weighted, _MA2Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = JMA(Open, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = JMA(Close, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = JMA(High, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = JMA(Low, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = JMA(Median, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = JMA(Typical, _MA2Period, _JMAPhase2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = JMA(Weighted, _MA2Period, _JMAPhase2).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = anaButterworthFilter(Open, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = anaButterworthFilter(Close, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = anaButterworthFilter(High, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = anaButterworthFilter(Low, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = anaButterworthFilter(Median, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = anaButterworthFilter(Typical, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = anaButterworthFilter(Weighted, _MA2Period, _ButterWorthFilterPoles2).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = anaGaussianFilter(Open, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = anaGaussianFilter(Close, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = anaGaussianFilter(High, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = anaGaussianFilter(Low, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = anaGaussianFilter(Median, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = anaGaussianFilter(Typical, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = anaGaussianFilter(Weighted, _MA2Period, _GaussianFilterPoles2).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA2PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA2Value = anaSuperSmootherFilter(Open, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA2Value = anaSuperSmootherFilter(Close, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA2Value = anaSuperSmootherFilter(High, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA2Value = anaSuperSmootherFilter(Low, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA2Value = anaSuperSmootherFilter(Median, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA2Value = anaSuperSmootherFilter(Typical, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA2Value = anaSuperSmootherFilter(Weighted, _MA2Period, _SuperSmootherFilterPoles2).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA3 Type
			switch (_MA3Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = SMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = SMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = SMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = SMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = SMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = SMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = SMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = EMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = EMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = EMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = EMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = EMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = EMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = EMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = HMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = HMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = HMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = HMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = HMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = HMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = HMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = TMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = TMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = TMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = TMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = TMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = TMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = TMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = TEMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = TEMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = TEMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = TEMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = TEMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = TEMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = TEMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = T3(Open, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = T3(Close, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = T3(High, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = T3(Low, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = T3(Median, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = T3(Typical, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = T3(Weighted, _MA3Period, _T3tCount3, _T3vFactor3).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = VWMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = VWMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = VWMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = VWMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = VWMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = VWMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = VWMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = WMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = WMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = WMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = WMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = WMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = WMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = WMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = ZLEMA(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = ZLEMA(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = ZLEMA(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = ZLEMA(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = ZLEMA(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = ZLEMA(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = ZLEMA(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = LLMA(Open, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = LLMA(Close, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = LLMA(High, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = LLMA(Low, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = LLMA(Median, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = LLMA(Typical, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = LLMA(Weighted, _MA3Period, _JMAPhase3).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = LinReg(Open, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = LinReg(Close, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = LinReg(High, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = LinReg(Low, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = LinReg(Median, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = LinReg(Typical, _MA3Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = LinReg(Weighted, _MA3Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = JMA(Open, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = JMA(Close, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = JMA(High, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = JMA(Low, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = JMA(Median, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = JMA(Typical, _MA3Period, _JMAPhase3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = JMA(Weighted, _MA3Period, _JMAPhase3).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = anaButterworthFilter(Open, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = anaButterworthFilter(Close, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = anaButterworthFilter(High, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = anaButterworthFilter(Low, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = anaButterworthFilter(Median, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = anaButterworthFilter(Typical, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = anaButterworthFilter(Weighted, _MA3Period, _ButterWorthFilterPoles3).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = anaGaussianFilter(Open, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = anaGaussianFilter(Close, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = anaGaussianFilter(High, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = anaGaussianFilter(Low, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = anaGaussianFilter(Median, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = anaGaussianFilter(Typical, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = anaGaussianFilter(Weighted, _MA3Period, _GaussianFilterPoles3).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA3PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA3Value = anaSuperSmootherFilter(Open, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA3Value = anaSuperSmootherFilter(Close, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA3Value = anaSuperSmootherFilter(High, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA3Value = anaSuperSmootherFilter(Low, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA3Value = anaSuperSmootherFilter(Median, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA3Value = anaSuperSmootherFilter(Typical, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA3Value = anaSuperSmootherFilter(Weighted, _MA3Period, _SuperSmootherFilterPoles3).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA4 Type
			switch (_MA4Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = SMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = SMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = SMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = SMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = SMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = SMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = SMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = EMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = EMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = EMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = EMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = EMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = EMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = EMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = HMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = HMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = HMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = HMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = HMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = HMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = HMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = TMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = TMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = TMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = TMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = TMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = TMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = TMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = TEMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = TEMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = TEMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = TEMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = TEMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = TEMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = TEMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = T3(Open, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = T3(Close, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = T3(High, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = T3(Low, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = T3(Median, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = T3(Typical, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = T3(Weighted, _MA4Period, _T3tCount4, _T3vFactor4).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = VWMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = VWMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = VWMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = VWMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = VWMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = VWMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = VWMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = WMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = WMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = WMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = WMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = WMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = WMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = WMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = ZLEMA(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = ZLEMA(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = ZLEMA(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = ZLEMA(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = ZLEMA(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = ZLEMA(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = ZLEMA(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = LLMA(Open, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = LLMA(Close, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = LLMA(High, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = LLMA(Low, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = LLMA(Median, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = LLMA(Typical, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = LLMA(Weighted, _MA4Period, _JMAPhase4).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = LinReg(Open, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = LinReg(Close, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = LinReg(High, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = LinReg(Low, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = LinReg(Median, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = LinReg(Typical, _MA4Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = LinReg(Weighted, _MA4Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = JMA(Open, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = JMA(Close, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = JMA(High, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = JMA(Low, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = JMA(Median, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = JMA(Typical, _MA4Period, _JMAPhase4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = JMA(Weighted, _MA4Period, _JMAPhase4).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = anaButterworthFilter(Open, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = anaButterworthFilter(Close, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = anaButterworthFilter(High, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = anaButterworthFilter(Low, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = anaButterworthFilter(Median, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = anaButterworthFilter(Typical, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = anaButterworthFilter(Weighted, _MA4Period, _ButterWorthFilterPoles4).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = anaGaussianFilter(Open, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = anaGaussianFilter(Close, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = anaGaussianFilter(High, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = anaGaussianFilter(Low, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = anaGaussianFilter(Median, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = anaGaussianFilter(Typical, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = anaGaussianFilter(Weighted, _MA4Period, _GaussianFilterPoles4).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA4PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA4Value = anaSuperSmootherFilter(Open, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA4Value = anaSuperSmootherFilter(Close, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA4Value = anaSuperSmootherFilter(High, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA4Value = anaSuperSmootherFilter(Low, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA4Value = anaSuperSmootherFilter(Median, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA4Value = anaSuperSmootherFilter(Typical, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA4Value = anaSuperSmootherFilter(Weighted, _MA4Period, _SuperSmootherFilterPoles4).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA5 Type
			switch (_MA5Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = SMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = SMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = SMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = SMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = SMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = SMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = SMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = EMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = EMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = EMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = EMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = EMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = EMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = EMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = HMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = HMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = HMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = HMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = HMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = HMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = HMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = TMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = TMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = TMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = TMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = TMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = TMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = TMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = TEMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = TEMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = TEMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = TEMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = TEMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = TEMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = TEMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = T3(Open, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = T3(Close, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = T3(High, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = T3(Low, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = T3(Median, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = T3(Typical, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = T3(Weighted, _MA5Period, _T3tCount5, _T3vFactor5).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = VWMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = VWMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = VWMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = VWMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = VWMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = VWMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = VWMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = WMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = WMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = WMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = WMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = WMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = WMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = WMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = ZLEMA(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = ZLEMA(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = ZLEMA(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = ZLEMA(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = ZLEMA(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = ZLEMA(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = ZLEMA(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = LLMA(Open, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = LLMA(Close, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = LLMA(High, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = LLMA(Low, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = LLMA(Median, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = LLMA(Typical, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = LLMA(Weighted, _MA5Period, _JMAPhase5).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = LinReg(Open, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = LinReg(Close, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = LinReg(High, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = LinReg(Low, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = LinReg(Median, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = LinReg(Typical, _MA5Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = LinReg(Weighted, _MA5Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = JMA(Open, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = JMA(Close, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = JMA(High, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = JMA(Low, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = JMA(Median, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = JMA(Typical, _MA5Period, _JMAPhase5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = JMA(Weighted, _MA5Period, _JMAPhase5).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = anaButterworthFilter(Open, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = anaButterworthFilter(Close, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = anaButterworthFilter(High, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = anaButterworthFilter(Low, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = anaButterworthFilter(Median, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = anaButterworthFilter(Typical, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = anaButterworthFilter(Weighted, _MA5Period, _ButterWorthFilterPoles5).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = anaGaussianFilter(Open, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = anaGaussianFilter(Close, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = anaGaussianFilter(High, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = anaGaussianFilter(Low, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = anaGaussianFilter(Median, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = anaGaussianFilter(Typical, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = anaGaussianFilter(Weighted, _MA5Period, _GaussianFilterPoles5).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA5PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA5Value = anaSuperSmootherFilter(Open, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA5Value = anaSuperSmootherFilter(Close, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA5Value = anaSuperSmootherFilter(High, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA5Value = anaSuperSmootherFilter(Low, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA5Value = anaSuperSmootherFilter(Median, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA5Value = anaSuperSmootherFilter(Typical, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA5Value = anaSuperSmootherFilter(Weighted, _MA5Period, _SuperSmootherFilterPoles5).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA6 Type
			switch (_MA6Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = SMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = SMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = SMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = SMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = SMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = SMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = SMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = EMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = EMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = EMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = EMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = EMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = EMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = EMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = HMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = HMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = HMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = HMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = HMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = HMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = HMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = TMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = TMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = TMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = TMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = TMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = TMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = TMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = TEMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = TEMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = TEMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = TEMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = TEMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = TEMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = TEMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = T3(Open, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = T3(Close, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = T3(High, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = T3(Low, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = T3(Median, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = T3(Typical, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = T3(Weighted, _MA6Period, _T3tCount6, _T3vFactor6).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = VWMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = VWMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = VWMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = VWMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = VWMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = VWMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = VWMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = WMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = WMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = WMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = WMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = WMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = WMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = WMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = ZLEMA(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = ZLEMA(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = ZLEMA(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = ZLEMA(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = ZLEMA(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = ZLEMA(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = ZLEMA(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = LLMA(Open, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = LLMA(Close, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = LLMA(High, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = LLMA(Low, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = LLMA(Median, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = LLMA(Typical, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = LLMA(Weighted, _MA6Period, _JMAPhase6).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = LinReg(Open, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = LinReg(Close, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = LinReg(High, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = LinReg(Low, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = LinReg(Median, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = LinReg(Typical, _MA6Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = LinReg(Weighted, _MA6Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = JMA(Open, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = JMA(Close, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = JMA(High, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = JMA(Low, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = JMA(Median, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = JMA(Typical, _MA6Period, _JMAPhase6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = JMA(Weighted, _MA6Period, _JMAPhase6).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = anaButterworthFilter(Open, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = anaButterworthFilter(Close, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = anaButterworthFilter(High, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = anaButterworthFilter(Low, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = anaButterworthFilter(Median, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = anaButterworthFilter(Typical, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = anaButterworthFilter(Weighted, _MA6Period, _ButterWorthFilterPoles6).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = anaGaussianFilter(Open, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = anaGaussianFilter(Close, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = anaGaussianFilter(High, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = anaGaussianFilter(Low, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = anaGaussianFilter(Median, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = anaGaussianFilter(Typical, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = anaGaussianFilter(Weighted, _MA6Period, _GaussianFilterPoles6).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA6PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA6Value = anaSuperSmootherFilter(Open, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA6Value = anaSuperSmootherFilter(Close, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA6Value = anaSuperSmootherFilter(High, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA6Value = anaSuperSmootherFilter(Low, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA6Value = anaSuperSmootherFilter(Median, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA6Value = anaSuperSmootherFilter(Typical, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA6Value = anaSuperSmootherFilter(Weighted, _MA6Period, _SuperSmootherFilterPoles6).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
			
			#region MA7 Type
			switch (_MA7Type)
			{
				case MAPackageMATypeV13.SMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = SMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = SMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = SMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = SMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = SMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = SMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = SMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.EMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = EMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = EMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = EMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = EMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = EMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = EMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = EMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.HMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = HMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = HMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = HMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = HMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = HMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = HMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = HMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = TMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = TMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = TMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = TMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = TMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = TMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = TMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.TEMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = TEMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = TEMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = TEMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = TEMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = TEMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = TEMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = TEMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.T3:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = T3(Open, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = T3(Close, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = T3(High, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = T3(Low, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = T3(Median, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = T3(Typical, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = T3(Weighted, _MA7Period, _T3tCount7, _T3vFactor7).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.VWMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = VWMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = VWMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = VWMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = VWMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = VWMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = VWMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = VWMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.WMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = WMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = WMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = WMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = WMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = WMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = WMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = WMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ZLEMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = ZLEMA(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = ZLEMA(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = ZLEMA(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = ZLEMA(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = ZLEMA(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = ZLEMA(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = ZLEMA(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LLMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = LLMA(Open, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = LLMA(Close, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = LLMA(High, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = LLMA(Low, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = LLMA(Median, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = LLMA(Typical, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = LLMA(Weighted, _MA7Period, _JMAPhase7).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.LinReg:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = LinReg(Open, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = LinReg(Close, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = LinReg(High, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = LinReg(Low, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = LinReg(Median, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = LinReg(Typical, _MA7Period).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = LinReg(Weighted, _MA7Period).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.JMA:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = JMA(Open, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = JMA(Close, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = JMA(High, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = JMA(Low, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = JMA(Median, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = JMA(Typical, _MA7Period, _JMAPhase7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = JMA(Weighted, _MA7Period, _JMAPhase7).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.ButterWorthFilter:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = anaButterworthFilter(Open, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = anaButterworthFilter(Close, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = anaButterworthFilter(High, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = anaButterworthFilter(Low, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = anaButterworthFilter(Median, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = anaButterworthFilter(Typical, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = anaButterworthFilter(Weighted, _MA7Period, _ButterWorthFilterPoles7).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.GaussianFilter:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = anaGaussianFilter(Open, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = anaGaussianFilter(Close, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = anaGaussianFilter(High, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = anaGaussianFilter(Low, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = anaGaussianFilter(Median, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = anaGaussianFilter(Typical, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = anaGaussianFilter(Weighted, _MA7Period, _GaussianFilterPoles7).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				case MAPackageMATypeV13.SuperSmootherFilter:
				{
					#region MA Price Type
					switch (_MA7PriceType)
					{
						case MAPackageMAPriceTypesV13.Open:
						{
							_MA7Value = anaSuperSmootherFilter(Open, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Close:
						{
							_MA7Value = anaSuperSmootherFilter(Close, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.High:
						{
							_MA7Value = anaSuperSmootherFilter(High, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Low:
						{
							_MA7Value = anaSuperSmootherFilter(Low, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Median:
						{
							_MA7Value = anaSuperSmootherFilter(Median, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Typical:
						{
							_MA7Value = anaSuperSmootherFilter(Typical, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						case MAPackageMAPriceTypesV13.Weighted:
						{
							_MA7Value = anaSuperSmootherFilter(Weighted, _MA7Period, _SuperSmootherFilterPoles7).Value;
							break;
						}
						default:
						{
							break;
						}
					}
					#endregion
					break;
				}
				default:
				{
					break;
				}
			}
			#endregion
		}
		#endregion
		
		#region OnBarUpdate
        protected override void OnBarUpdate()
        {
			#region Init
			if(CurrentBar < 5)
				return;
			#endregion
			
			#region MA1
			if(_UseMA1)
			{
				MA1.Set(_MA1Value[0]);
				if(_MA1Value[0] > _MA1Value[1])
				{
					PlotColors[0][0] = Color.FromArgb(_MA1RisingColorOpacity, _MA1RisingColor);
					MA1Direction = MADirectionTypes.Rising;
				}
				else if(_MA1Value[0] < _MA1Value[1])
				{
					PlotColors[0][0] = Color.FromArgb(_MA1FallingColorOpacity, _MA1FallingColor);
					MA1Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA1Direction == MADirectionTypes.Rising)
						PlotColors[0][0] = Color.FromArgb(_MA1RisingColorOpacity, _MA1RisingColor);
					else
					{
						PlotColors[0][0] = Color.FromArgb(_MA1FallingColorOpacity, _MA1FallingColor);
						MA1Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
			
			#region MA2
			if(_UseMA2)
			{
				MA2.Set(_MA2Value[0]);
				if(_MA2Value[0] > _MA2Value[1])
				{
					PlotColors[1][0] = Color.FromArgb(_MA2RisingColorOpacity, _MA2RisingColor);
					MA2Direction = MADirectionTypes.Rising;
				}
				else if(_MA2Value[0] < _MA2Value[1])
				{
					PlotColors[1][0] = Color.FromArgb(_MA2FallingColorOpacity, _MA2FallingColor);
					MA2Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA2Direction == MADirectionTypes.Rising)
						PlotColors[1][0] = Color.FromArgb(_MA2RisingColorOpacity, _MA2RisingColor);
					else
					{
						PlotColors[1][0] = Color.FromArgb(_MA2FallingColorOpacity, _MA2FallingColor);
						MA2Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
			
			#region MA3
			if(_UseMA3)
			{
				MA3.Set(_MA3Value[0]);
				if(_MA3Value[0] > _MA3Value[1])
				{
					PlotColors[2][0] = Color.FromArgb(_MA3RisingColorOpacity, _MA3RisingColor);
					MA3Direction = MADirectionTypes.Rising;
				}
				else if(_MA3Value[0] < _MA3Value[1])
				{
					PlotColors[2][0] = Color.FromArgb(_MA3FallingColorOpacity, _MA3FallingColor);
					MA3Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA3Direction == MADirectionTypes.Rising)
						PlotColors[2][0] = Color.FromArgb(_MA3RisingColorOpacity, _MA3RisingColor);
					else
					{
						PlotColors[2][0] = Color.FromArgb(_MA3FallingColorOpacity, _MA3FallingColor);
						MA3Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
			
			#region MA4
			if(_UseMA4)
			{
				MA4.Set(_MA4Value[0]);
				if(_MA4Value[0] > _MA4Value[1])
				{
					PlotColors[3][0] = Color.FromArgb(_MA4RisingColorOpacity, _MA4RisingColor);
					MA4Direction = MADirectionTypes.Rising;
				}
				else if(_MA4Value[0] < _MA4Value[1])
				{
					PlotColors[3][0] = Color.FromArgb(_MA4FallingColorOpacity, _MA4FallingColor);
					MA4Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA4Direction == MADirectionTypes.Rising)
						PlotColors[3][0] = Color.FromArgb(_MA4RisingColorOpacity, _MA4RisingColor);
					else
					{
						PlotColors[3][0] = Color.FromArgb(_MA4FallingColorOpacity, _MA4FallingColor);
						MA4Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
			
			#region MA5
			if(_UseMA5)
			{
				MA5.Set(_MA5Value[0]);
				if(_MA5Value[0] > _MA5Value[1])
				{
					PlotColors[4][0] = Color.FromArgb(_MA5RisingColorOpacity, _MA5RisingColor);
					MA5Direction = MADirectionTypes.Rising;
				}
				else if(_MA5Value[0] < _MA5Value[1])
				{
					PlotColors[4][0] = Color.FromArgb(_MA5FallingColorOpacity, _MA5FallingColor);
					MA5Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA5Direction == MADirectionTypes.Rising)
						PlotColors[4][0] = Color.FromArgb(_MA5RisingColorOpacity, _MA5RisingColor);
					else
					{
						PlotColors[4][0] = Color.FromArgb(_MA5FallingColorOpacity, _MA5FallingColor);
						MA5Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
			
			#region MA6
			if(_UseMA6)
			{
				MA6.Set(_MA6Value[0]);
				if(_MA6Value[0] > _MA6Value[1])
				{
					PlotColors[5][0] = Color.FromArgb(_MA6RisingColorOpacity, _MA6RisingColor);
					MA6Direction = MADirectionTypes.Rising;
				}
				else if(_MA6Value[0] < _MA6Value[1])
				{
					PlotColors[5][0] = Color.FromArgb(_MA6FallingColorOpacity, _MA6FallingColor);
					MA6Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA6Direction == MADirectionTypes.Rising)
						PlotColors[5][0] = Color.FromArgb(_MA6RisingColorOpacity, _MA6RisingColor);
					else
					{
						PlotColors[5][0] = Color.FromArgb(_MA6FallingColorOpacity, _MA6FallingColor);
						MA6Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
			
			#region MA7
			if(_UseMA7)
			{
				MA7.Set(_MA7Value[0]);
				if(_MA7Value[0] > _MA7Value[1])
				{
					PlotColors[6][0] = Color.FromArgb(_MA7RisingColorOpacity, _MA7RisingColor);
					MA7Direction = MADirectionTypes.Rising;
				}
				else if(_MA7Value[0] < _MA7Value[1])
				{
					PlotColors[6][0] = Color.FromArgb(_MA7FallingColorOpacity, _MA7FallingColor);
					MA7Direction = MADirectionTypes.Falling;
				}
				else
				{
					if(MA7Direction == MADirectionTypes.Rising)
						PlotColors[6][0] = Color.FromArgb(_MA7RisingColorOpacity, _MA7RisingColor);
					else
					{
						PlotColors[6][0] = Color.FromArgb(_MA7FallingColorOpacity, _MA7FallingColor);
						MA7Direction = MADirectionTypes.Falling;
					}
				}
			}
			#endregion
        }
		#endregion
								
        #region Properties
		#region Values
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA1
		{
			get { return Values[0]; }
		}
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA2
		{
			get { return Values[1]; }
		}
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA3
		{
			get { return Values[2]; }
		}
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA4
		{
			get { return Values[3]; }
		}
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA5
		{
			get { return Values[4]; }
		}
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA6
		{
			get { return Values[5]; }
		}
		[Browsable(false)]	
		[XmlIgnore()]		
		public DataSeries MA7
		{
			get { return Values[6]; }
		}
		#endregion
		
		#region MA1 Parameters
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Enabled")]
		[Gui.Design.DisplayName("1.1.1 MA1 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA1
		{
			get { return _UseMA1; }
			set { _UseMA1 = value; }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Type")]
		[Gui.Design.DisplayName("1.1.2 MA1 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA1Type
		{
			get { return _MA1Type; }
			set { _MA1Type = value; }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("1.1.2.1 MA1 JMA/LLMA Phase")]
		public int JMAPhase1
		{
			get { return _JMAPhase1; }
			set { _JMAPhase1 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 T3 Count")]
		[Gui.Design.DisplayName("1.1.2.2 MA1 T3 Count")]
		public int T3tCount1
		{
			get { return _T3tCount1; }
			set { _T3tCount1 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 T3 vFactor")]
		[Gui.Design.DisplayName("1.1.2.3 MA1 T3 vFactor")]
		public double T3vFactor1
		{
			get { return _T3vFactor1; }
			set { _T3vFactor1 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("1.1.2.4 MA1 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles1
		{
			get { return _ButterWorthFilterPoles1; }
			set { _ButterWorthFilterPoles1 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("1.1.2.5 MA1 Gaussian Filter Poles")]
		public int GaussianFilterPoles1
		{
			get { return _GaussianFilterPoles1; }
			set { _GaussianFilterPoles1 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("1.1.2.6 MA1 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles1
		{
			get { return _SuperSmootherFilterPoles1; }
			set { _SuperSmootherFilterPoles1 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Period")]
		[Gui.Design.DisplayName("1.1.3 MA1 Period")]
		public int MA1Period
		{
			get { return _MA1Period; }
			set { _MA1Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Price Type")]
		[Gui.Design.DisplayName("1.1.4 MA1 Price Type")]
		public MAPackageMAPriceTypesV13 MA1PriceType
		{
			get { return _MA1PriceType; }
			set { _MA1PriceType = value; }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Rising Color")]
		[Gui.Design.DisplayName("1.2.1 MA1 Rising Color")]
		public Color MA1RisingColor
		{
			get { return _MA1RisingColor; }
			set { _MA1RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA1RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA1RisingColor); }
			set { _MA1RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Rising Color Opacity")]
		[Gui.Design.DisplayName("1.2.2 MA1 Rising Color Opacity")]
		public int MA1RisingColorOpacity
		{
			get { return _MA1RisingColorOpacity; }
			set { _MA1RisingColorOpacity = value; }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Falling Color")]
		[Gui.Design.DisplayName("1.2.3 MA1 Falling Color")]
		public Color MA1FallingColor
		{
			get { return _MA1FallingColor; }
			set { _MA1FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA1FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA1FallingColor); }
			set { _MA1FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Falling Color Opacity")]
		[Gui.Design.DisplayName("1.2.4 MA1 Falling Color Opacity")]
		public int MA1FallingColorOpacity
		{
			get { return _MA1FallingColorOpacity; }
			set { _MA1FallingColorOpacity = value; }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Line Width")]
		[Gui.Design.DisplayName("1.2.5 MA line default width")]
		public int MA1LineWidth
		{
			get { return _MA1LineWidth; }
			set { _MA1LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Line Dash Style")]
		[Gui.Design.DisplayName("1.2.6 MA1 Line Dash Style")]
		public MAPackageDashStyleV13 MA1LineDashStyle
		{
			get { return _MA1LineDashStyle; }
			set { _MA1LineDashStyle = value; }
		}
		
		[GridCategory("1. MA1 Parameters")]
		[Description("MA1 Line Plot Style")]
		[Gui.Design.DisplayName("1.2.7 MA1 Line Plot Style")]
		public MAPackagePlotStyleV13 MA1LinePlotStyle
		{
			get { return _MA1LinePlotStyle; }
			set { _MA1LinePlotStyle = value; }
		}
		#endregion
		
		#region MA2 Parameters
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Enabled")]
		[Gui.Design.DisplayName("2.1.1 MA2 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA2
		{
			get { return _UseMA2; }
			set { _UseMA2 = value; }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Type")]
		[Gui.Design.DisplayName("2.1.2 MA2 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA2Type
		{
			get { return _MA2Type; }
			set { _MA2Type = value; }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("2.1.2.1 MA2 JMA/LLMA Phase")]
		public int JMAPhase2
		{
			get { return _JMAPhase2; }
			set { _JMAPhase2 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 T3 Count")]
		[Gui.Design.DisplayName("2.1.2.2 MA2 T3 Count")]
		public int T3tCount2
		{
			get { return _T3tCount2; }
			set { _T3tCount2 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 T3 vFactor")]
		[Gui.Design.DisplayName("2.1.2.3 MA2 T3 vFactor")]
		public double T3vFactor2
		{
			get { return _T3vFactor2; }
			set { _T3vFactor2 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("2.1.2.4 MA2 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles2
		{
			get { return _ButterWorthFilterPoles2; }
			set { _ButterWorthFilterPoles2 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("2.1.2.5 MA2 Gaussian Filter Poles")]
		public int GaussianFilterPoles2
		{
			get { return _GaussianFilterPoles2; }
			set { _GaussianFilterPoles2 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("2.1.2.6 MA2 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles2
		{
			get { return _SuperSmootherFilterPoles2; }
			set { _SuperSmootherFilterPoles2 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Period")]
		[Gui.Design.DisplayName("2.1.3 MA2 Period")]
		public int MA2Period
		{
			get { return _MA2Period; }
			set { _MA2Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Price Type")]
		[Gui.Design.DisplayName("2.1.4 MA2 Price Type")]
		public MAPackageMAPriceTypesV13 MA2PriceType
		{
			get { return _MA2PriceType; }
			set { _MA2PriceType = value; }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Rising Color")]
		[Gui.Design.DisplayName("2.2.1 MA2 Rising Color")]
		public Color MA2RisingColor
		{
			get { return _MA2RisingColor; }
			set { _MA2RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA2RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA2RisingColor); }
			set { _MA2RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Rising Color Opacity")]
		[Gui.Design.DisplayName("2.2.2 MA2 Rising Color Opacity")]
		public int MA2RisingColorOpacity
		{
			get { return _MA2RisingColorOpacity; }
			set { _MA2RisingColorOpacity = value; }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Falling Color")]
		[Gui.Design.DisplayName("2.2.3 MA2 Falling Color")]
		public Color MA2FallingColor
		{
			get { return _MA2FallingColor; }
			set { _MA2FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA2FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA2FallingColor); }
			set { _MA2FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Falling Color Opacity")]
		[Gui.Design.DisplayName("2.2.4 MA2 Falling Color Opacity")]
		public int MA2FallingColorOpacity
		{
			get { return _MA2FallingColorOpacity; }
			set { _MA2FallingColorOpacity = value; }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Line Width")]
		[Gui.Design.DisplayName("2.2.5 MA line default width")]
		public int MA2LineWidth
		{
			get { return _MA2LineWidth; }
			set { _MA2LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Line Dash Style")]
		[Gui.Design.DisplayName("2.2.6 MA2 Line Dash Style")]
		public MAPackageDashStyleV13 MA2LineDashStyle
		{
			get { return _MA2LineDashStyle; }
			set { _MA2LineDashStyle = value; }
		}
		
		[GridCategory("2. MA2 Parameters")]
		[Description("MA2 Line Plot Style")]
		[Gui.Design.DisplayName("2.2.7 MA2 Line Plot Style")]
		public MAPackagePlotStyleV13 MA2LinePlotStyle
		{
			get { return _MA2LinePlotStyle; }
			set { _MA2LinePlotStyle = value; }
		}
		#endregion
		
		#region MA3 Parameters
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Enabled")]
		[Gui.Design.DisplayName("3.1.1 MA3 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA3
		{
			get { return _UseMA3; }
			set { _UseMA3 = value; }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Type")]
		[Gui.Design.DisplayName("3.1.2 MA3 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA3Type
		{
			get { return _MA3Type; }
			set { _MA3Type = value; }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("3.1.2.1 MA3 JMA/LLMA Phase")]
		public int JMAPhase3
		{
			get { return _JMAPhase3; }
			set { _JMAPhase3 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 T3 Count")]
		[Gui.Design.DisplayName("3.1.2.2 MA3 T3 Count")]
		public int T3tCount3
		{
			get { return _T3tCount3; }
			set { _T3tCount3 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 T3 vFactor")]
		[Gui.Design.DisplayName("3.1.2.3 MA3 T3 vFactor")]
		public double T3vFactor3
		{
			get { return _T3vFactor3; }
			set { _T3vFactor3 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("3.1.2.4 MA3 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles3
		{
			get { return _ButterWorthFilterPoles3; }
			set { _ButterWorthFilterPoles3 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("3.1.2.5 MA3 Gaussian Filter Poles")]
		public int GaussianFilterPoles3
		{
			get { return _GaussianFilterPoles3; }
			set { _GaussianFilterPoles3 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("3.1.2.6 MA3 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles3
		{
			get { return _SuperSmootherFilterPoles3; }
			set { _SuperSmootherFilterPoles3 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Period")]
		[Gui.Design.DisplayName("3.1.3 MA3 Period")]
		public int MA3Period
		{
			get { return _MA3Period; }
			set { _MA3Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Price Type")]
		[Gui.Design.DisplayName("3.1.4 MA3 Price Type")]
		public MAPackageMAPriceTypesV13 MA3PriceType
		{
			get { return _MA3PriceType; }
			set { _MA3PriceType = value; }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Rising Color")]
		[Gui.Design.DisplayName("3.2.1 MA3 Rising Color")]
		public Color MA3RisingColor
		{
			get { return _MA3RisingColor; }
			set { _MA3RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA3RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA3RisingColor); }
			set { _MA3RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Rising Color Opacity")]
		[Gui.Design.DisplayName("3.2.2 MA3 Rising Color Opacity")]
		public int MA3RisingColorOpacity
		{
			get { return _MA3RisingColorOpacity; }
			set { _MA3RisingColorOpacity = value; }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Falling Color")]
		[Gui.Design.DisplayName("3.2.3 MA3 Falling Color")]
		public Color MA3FallingColor
		{
			get { return _MA3FallingColor; }
			set { _MA3FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA3FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA3FallingColor); }
			set { _MA3FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Falling Color Opacity")]
		[Gui.Design.DisplayName("3.2.4 MA3 Falling Color Opacity")]
		public int MA3FallingColorOpacity
		{
			get { return _MA3FallingColorOpacity; }
			set { _MA3FallingColorOpacity = value; }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Line Width")]
		[Gui.Design.DisplayName("3.2.5 MA line default width")]
		public int MA3LineWidth
		{
			get { return _MA3LineWidth; }
			set { _MA3LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Line Dash Style")]
		[Gui.Design.DisplayName("3.2.6 MA3 Line Dash Style")]
		public MAPackageDashStyleV13 MA3LineDashStyle
		{
			get { return _MA3LineDashStyle; }
			set { _MA3LineDashStyle = value; }
		}
		
		[GridCategory("3. MA3 Parameters")]
		[Description("MA3 Line Plot Style")]
		[Gui.Design.DisplayName("3.2.7 MA3 Line Plot Style")]
		public MAPackagePlotStyleV13 MA3LinePlotStyle
		{
			get { return _MA3LinePlotStyle; }
			set { _MA3LinePlotStyle = value; }
		}
		#endregion
		
		#region MA4 Parameters
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Enabled")]
		[Gui.Design.DisplayName("4.1.1 MA4 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA4
		{
			get { return _UseMA4; }
			set { _UseMA4 = value; }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Type")]
		[Gui.Design.DisplayName("4.1.2 MA4 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA4Type
		{
			get { return _MA4Type; }
			set { _MA4Type = value; }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("4.1.2.1 MA4 JMA/LLMA Phase")]
		public int JMAPhase4
		{
			get { return _JMAPhase4; }
			set { _JMAPhase4 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 T3 Count")]
		[Gui.Design.DisplayName("4.1.2.2 MA4 T3 Count")]
		public int T3tCount4
		{
			get { return _T3tCount4; }
			set { _T3tCount4 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 T3 vFactor")]
		[Gui.Design.DisplayName("4.1.2.3 MA4 T3 vFactor")]
		public double T3vFactor4
		{
			get { return _T3vFactor4; }
			set { _T3vFactor4 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("4.1.2.4 MA4 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles4
		{
			get { return _ButterWorthFilterPoles4; }
			set { _ButterWorthFilterPoles4 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("4.1.2.5 MA4 Gaussian Filter Poles")]
		public int GaussianFilterPoles4
		{
			get { return _GaussianFilterPoles4; }
			set { _GaussianFilterPoles4 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("4.1.2.6 MA4 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles4
		{
			get { return _SuperSmootherFilterPoles4; }
			set { _SuperSmootherFilterPoles4 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Period")]
		[Gui.Design.DisplayName("4.1.3 MA4 Period")]
		public int MA4Period
		{
			get { return _MA4Period; }
			set { _MA4Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Price Type")]
		[Gui.Design.DisplayName("4.1.4 MA4 Price Type")]
		public MAPackageMAPriceTypesV13 MA4PriceType
		{
			get { return _MA4PriceType; }
			set { _MA4PriceType = value; }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Rising Color")]
		[Gui.Design.DisplayName("4.2.1 MA4 Rising Color")]
		public Color MA4RisingColor
		{
			get { return _MA4RisingColor; }
			set { _MA4RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA4RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA4RisingColor); }
			set { _MA4RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Rising Color Opacity")]
		[Gui.Design.DisplayName("4.2.2 MA4 Rising Color Opacity")]
		public int MA4RisingColorOpacity
		{
			get { return _MA4RisingColorOpacity; }
			set { _MA4RisingColorOpacity = value; }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Falling Color")]
		[Gui.Design.DisplayName("4.2.3 MA4 Falling Color")]
		public Color MA4FallingColor
		{
			get { return _MA4FallingColor; }
			set { _MA4FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA4FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA4FallingColor); }
			set { _MA4FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Falling Color Opacity")]
		[Gui.Design.DisplayName("4.2.4 MA4 Falling Color Opacity")]
		public int MA4FallingColorOpacity
		{
			get { return _MA4FallingColorOpacity; }
			set { _MA4FallingColorOpacity = value; }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Line Width")]
		[Gui.Design.DisplayName("4.2.5 MA line default width")]
		public int MA4LineWidth
		{
			get { return _MA4LineWidth; }
			set { _MA4LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Line Dash Style")]
		[Gui.Design.DisplayName("4.2.6 MA4 Line Dash Style")]
		public MAPackageDashStyleV13 MA4LineDashStyle
		{
			get { return _MA4LineDashStyle; }
			set { _MA4LineDashStyle = value; }
		}
		
		[GridCategory("4. MA4 Parameters")]
		[Description("MA4 Line Plot Style")]
		[Gui.Design.DisplayName("4.2.7 MA4 Line Plot Style")]
		public MAPackagePlotStyleV13 MA4LinePlotStyle
		{
			get { return _MA4LinePlotStyle; }
			set { _MA4LinePlotStyle = value; }
		}
		#endregion
		
		#region MA5 Parameters
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Enabled")]
		[Gui.Design.DisplayName("5.1.1 MA5 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA5
		{
			get { return _UseMA5; }
			set { _UseMA5 = value; }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Type")]
		[Gui.Design.DisplayName("5.1.2 MA5 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA5Type
		{
			get { return _MA5Type; }
			set { _MA5Type = value; }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("5.1.2.1 MA5 JMA/LLMA Phase")]
		public int JMAPhase5
		{
			get { return _JMAPhase5; }
			set { _JMAPhase5 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 T3 Count")]
		[Gui.Design.DisplayName("5.1.2.2 MA5 T3 Count")]
		public int T3tCount5
		{
			get { return _T3tCount5; }
			set { _T3tCount5 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 T3 vFactor")]
		[Gui.Design.DisplayName("5.1.2.3 MA5 T3 vFactor")]
		public double T3vFactor5
		{
			get { return _T3vFactor5; }
			set { _T3vFactor5 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("5.1.2.4 MA5 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles5
		{
			get { return _ButterWorthFilterPoles5; }
			set { _ButterWorthFilterPoles5 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("5.1.2.5 MA5 Gaussian Filter Poles")]
		public int GaussianFilterPoles5
		{
			get { return _GaussianFilterPoles5; }
			set { _GaussianFilterPoles5 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("5.1.2.6 MA5 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles5
		{
			get { return _SuperSmootherFilterPoles5; }
			set { _SuperSmootherFilterPoles5 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Period")]
		[Gui.Design.DisplayName("5.1.3 MA5 Period")]
		public int MA5Period
		{
			get { return _MA5Period; }
			set { _MA5Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Price Type")]
		[Gui.Design.DisplayName("5.1.4 MA5 Price Type")]
		public MAPackageMAPriceTypesV13 MA5PriceType
		{
			get { return _MA5PriceType; }
			set { _MA5PriceType = value; }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Rising Color")]
		[Gui.Design.DisplayName("5.2.1 MA5 Rising Color")]
		public Color MA5RisingColor
		{
			get { return _MA5RisingColor; }
			set { _MA5RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA5RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA5RisingColor); }
			set { _MA5RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Rising Color Opacity")]
		[Gui.Design.DisplayName("5.2.2 MA5 Rising Color Opacity")]
		public int MA5RisingColorOpacity
		{
			get { return _MA5RisingColorOpacity; }
			set { _MA5RisingColorOpacity = value; }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Falling Color")]
		[Gui.Design.DisplayName("5.2.3 MA5 Falling Color")]
		public Color MA5FallingColor
		{
			get { return _MA5FallingColor; }
			set { _MA5FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA5FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA5FallingColor); }
			set { _MA5FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Falling Color Opacity")]
		[Gui.Design.DisplayName("5.2.4 MA5 Falling Color Opacity")]
		public int MA5FallingColorOpacity
		{
			get { return _MA5FallingColorOpacity; }
			set { _MA5FallingColorOpacity = value; }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Line Width")]
		[Gui.Design.DisplayName("5.2.5 MA line default width")]
		public int MA5LineWidth
		{
			get { return _MA5LineWidth; }
			set { _MA5LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Line Dash Style")]
		[Gui.Design.DisplayName("5.2.6 MA5 Line Dash Style")]
		public MAPackageDashStyleV13 MA5LineDashStyle
		{
			get { return _MA5LineDashStyle; }
			set { _MA5LineDashStyle = value; }
		}
		
		[GridCategory("5. MA5 Parameters")]
		[Description("MA5 Line Plot Style")]
		[Gui.Design.DisplayName("5.2.7 MA5 Line Plot Style")]
		public MAPackagePlotStyleV13 MA5LinePlotStyle
		{
			get { return _MA5LinePlotStyle; }
			set { _MA5LinePlotStyle = value; }
		}
		#endregion
		
		#region MA6 Parameters
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Enabled")]
		[Gui.Design.DisplayName("6.1.1 MA6 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA6
		{
			get { return _UseMA6; }
			set { _UseMA6 = value; }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Type")]
		[Gui.Design.DisplayName("6.1.2 MA6 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA6Type
		{
			get { return _MA6Type; }
			set { _MA6Type = value; }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("6.1.2.1 MA6 JMA/LLMA Phase")]
		public int JMAPhase6
		{
			get { return _JMAPhase6; }
			set { _JMAPhase6 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 T3 Count")]
		[Gui.Design.DisplayName("6.1.2.2 MA6 T3 Count")]
		public int T3tCount6
		{
			get { return _T3tCount6; }
			set { _T3tCount6 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 T3 vFactor")]
		[Gui.Design.DisplayName("6.1.2.3 MA6 T3 vFactor")]
		public double T3vFactor6
		{
			get { return _T3vFactor6; }
			set { _T3vFactor6 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("6.1.2.4 MA6 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles6
		{
			get { return _ButterWorthFilterPoles6; }
			set { _ButterWorthFilterPoles6 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("6.1.2.5 MA6 Gaussian Filter Poles")]
		public int GaussianFilterPoles6
		{
			get { return _GaussianFilterPoles6; }
			set { _GaussianFilterPoles6 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("6.1.2.6 MA6 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles6
		{
			get { return _SuperSmootherFilterPoles6; }
			set { _SuperSmootherFilterPoles6 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Period")]
		[Gui.Design.DisplayName("6.1.3 MA6 Period")]
		public int MA6Period
		{
			get { return _MA6Period; }
			set { _MA6Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Price Type")]
		[Gui.Design.DisplayName("6.1.4 MA6 Price Type")]
		public MAPackageMAPriceTypesV13 MA6PriceType
		{
			get { return _MA6PriceType; }
			set { _MA6PriceType = value; }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Rising Color")]
		[Gui.Design.DisplayName("6.2.1 MA6 Rising Color")]
		public Color MA6RisingColor
		{
			get { return _MA6RisingColor; }
			set { _MA6RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA6RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA6RisingColor); }
			set { _MA6RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Rising Color Opacity")]
		[Gui.Design.DisplayName("6.2.2 MA6 Rising Color Opacity")]
		public int MA6RisingColorOpacity
		{
			get { return _MA6RisingColorOpacity; }
			set { _MA6RisingColorOpacity = value; }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Falling Color")]
		[Gui.Design.DisplayName("6.2.3 MA6 Falling Color")]
		public Color MA6FallingColor
		{
			get { return _MA6FallingColor; }
			set { _MA6FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA6FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA6FallingColor); }
			set { _MA6FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Falling Color Opacity")]
		[Gui.Design.DisplayName("6.2.4 MA6 Falling Color Opacity")]
		public int MA6FallingColorOpacity
		{
			get { return _MA6FallingColorOpacity; }
			set { _MA6FallingColorOpacity = value; }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Line Width")]
		[Gui.Design.DisplayName("6.2.5 MA line default width")]
		public int MA6LineWidth
		{
			get { return _MA6LineWidth; }
			set { _MA6LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Line Dash Style")]
		[Gui.Design.DisplayName("6.2.6 MA6 Line Dash Style")]
		public MAPackageDashStyleV13 MA6LineDashStyle
		{
			get { return _MA6LineDashStyle; }
			set { _MA6LineDashStyle = value; }
		}
		
		[GridCategory("6. MA6 Parameters")]
		[Description("MA6 Line Plot Style")]
		[Gui.Design.DisplayName("6.2.7 MA6 Line Plot Style")]
		public MAPackagePlotStyleV13 MA6LinePlotStyle
		{
			get { return _MA6LinePlotStyle; }
			set { _MA6LinePlotStyle = value; }
		}
		#endregion
		
		#region MA7 Parameters
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Enabled")]
		[Gui.Design.DisplayName("7.1.1 MA7 Enabled")]
		[RefreshProperties(RefreshProperties.All)]
		public bool UseMA7
		{
			get { return _UseMA7; }
			set { _UseMA7 = value; }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Type")]
		[Gui.Design.DisplayName("7.1.2 MA7 Type")]
		[RefreshProperties(RefreshProperties.All)]
		public MAPackageMATypeV13 MA7Type
		{
			get { return _MA7Type; }
			set { _MA7Type = value; }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 JMA/LLMA Phase")]
		[Gui.Design.DisplayName("7.1.2.1 MA7 JMA/LLMA Phase")]
		public int JMAPhase7
		{
			get { return _JMAPhase7; }
			set { _JMAPhase7 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 T3 Count")]
		[Gui.Design.DisplayName("7.1.2.2 MA7 T3 Count")]
		public int T3tCount7
		{
			get { return _T3tCount7; }
			set { _T3tCount7 = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 T3 vFactor")]
		[Gui.Design.DisplayName("7.1.2.3 MA7 T3 vFactor")]
		public double T3vFactor7
		{
			get { return _T3vFactor7; }
			set { _T3vFactor7 = Math.Min(1000,Math.Max(0,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 ButterWorth Filter Poles")]
		[Gui.Design.DisplayName("7.1.2.4 MA7 ButterWorth Filter Poles")]
		public int ButterWorthFilterPoles7
		{
			get { return _ButterWorthFilterPoles7; }
			set { _ButterWorthFilterPoles7 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Gaussian Filter Poles")]
		[Gui.Design.DisplayName("7.1.2.5 MA7 Gaussian Filter Poles")]
		public int GaussianFilterPoles7
		{
			get { return _GaussianFilterPoles7; }
			set { _GaussianFilterPoles7 = Math.Min(4,Math.Max(1,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Super Smoother Filter Poles")]
		[Gui.Design.DisplayName("7.1.2.6 MA7 Super Smoother Filter Poles")]
		public int SuperSmootherFilterPoles7
		{
			get { return _SuperSmootherFilterPoles7; }
			set { _SuperSmootherFilterPoles7 = Math.Min(3,Math.Max(2,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Period")]
		[Gui.Design.DisplayName("7.1.3 MA7 Period")]
		public int MA7Period
		{
			get { return _MA7Period; }
			set { _MA7Period = Math.Min(1000,Math.Max(1,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Price Type")]
		[Gui.Design.DisplayName("7.1.4 MA7 Price Type")]
		public MAPackageMAPriceTypesV13 MA7PriceType
		{
			get { return _MA7PriceType; }
			set { _MA7PriceType = value; }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Rising Color")]
		[Gui.Design.DisplayName("7.2.1 MA7 Rising Color")]
		public Color MA7RisingColor
		{
			get { return _MA7RisingColor; }
			set { _MA7RisingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA7RisingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA7RisingColor); }
			set { _MA7RisingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Rising Color Opacity")]
		[Gui.Design.DisplayName("7.2.2 MA7 Rising Color Opacity")]
		public int MA7RisingColorOpacity
		{
			get { return _MA7RisingColorOpacity; }
			set { _MA7RisingColorOpacity = value; }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Falling Color")]
		[Gui.Design.DisplayName("7.2.3 MA7 Falling Color")]
		public Color MA7FallingColor
		{
			get { return _MA7FallingColor; }
			set { _MA7FallingColor = value; }
		}
		//Serialize
		[Browsable(false)]
		public string MA7FallingColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_MA7FallingColor); }
			set { _MA7FallingColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Falling Color Opacity")]
		[Gui.Design.DisplayName("7.2.4 MA7 Falling Color Opacity")]
		public int MA7FallingColorOpacity
		{
			get { return _MA7FallingColorOpacity; }
			set { _MA7FallingColorOpacity = value; }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Line Width")]
		[Gui.Design.DisplayName("7.2.5 MA line default width")]
		public int MA7LineWidth
		{
			get { return _MA7LineWidth; }
			set { _MA7LineWidth = Math.Min(255,Math.Max(1,value)); }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Line Dash Style")]
		[Gui.Design.DisplayName("7.2.6 MA7 Line Dash Style")]
		public MAPackageDashStyleV13 MA7LineDashStyle
		{
			get { return _MA7LineDashStyle; }
			set { _MA7LineDashStyle = value; }
		}
		
		[GridCategory("7. MA7 Parameters")]
		[Description("MA7 Line Plot Style")]
		[Gui.Design.DisplayName("7.2.7 MA7 Line Plot Style")]
		public MAPackagePlotStyleV13 MA7LinePlotStyle
		{
			get { return _MA7LinePlotStyle; }
			set { _MA7LinePlotStyle = value; }
		}
		#endregion
		
		#region Dynamic Property
        private void ModifyProperties(PropertyDescriptorCollection col)
        {
			#region MA1
			if(!UseMA1)
			{
				col.Remove(col.Find("MA1Type", true));
				col.Remove(col.Find("MA1Period", true));
				col.Remove(col.Find("MA1PriceType", true));
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
				col.Remove(col.Find("MA1RisingColor", true));
				col.Remove(col.Find("MA1RisingColorOpacity", true));
				col.Remove(col.Find("MA1FallingColor", true));
				col.Remove(col.Find("MA1FallingColorOpacity", true));
				col.Remove(col.Find("MA1LineWidth", true));
				col.Remove(col.Find("MA1LineDashStyle", true));
				col.Remove(col.Find("MA1LinePlotStyle", true));
			}
			
			if(_MA1Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("SuperSmootherFilterPoles1", true));
			}
			else if(_MA1Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase1", true));
				col.Remove(col.Find("T3tCount1", true));
				col.Remove(col.Find("T3vFactor1", true));
				col.Remove(col.Find("ButterWorthFilterPoles1", true));
				col.Remove(col.Find("GaussianFilterPoles1", true));
			}
			#endregion
			
			#region MA2
			if(!UseMA2)
			{
				col.Remove(col.Find("MA2Type", true));
				col.Remove(col.Find("MA2Period", true));
				col.Remove(col.Find("MA2PriceType", true));
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
				col.Remove(col.Find("MA2RisingColor", true));
				col.Remove(col.Find("MA2RisingColorOpacity", true));
				col.Remove(col.Find("MA2FallingColor", true));
				col.Remove(col.Find("MA2FallingColorOpacity", true));
				col.Remove(col.Find("MA2LineWidth", true));
				col.Remove(col.Find("MA2LineDashStyle", true));
				col.Remove(col.Find("MA2LinePlotStyle", true));
			}
			
			if(_MA2Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("SuperSmootherFilterPoles2", true));
			}
			else if(_MA2Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase2", true));
				col.Remove(col.Find("T3tCount2", true));
				col.Remove(col.Find("T3vFactor2", true));
				col.Remove(col.Find("ButterWorthFilterPoles2", true));
				col.Remove(col.Find("GaussianFilterPoles2", true));
			}
			#endregion
			
			#region MA3
			if(!UseMA3)
			{
				col.Remove(col.Find("MA3Type", true));
				col.Remove(col.Find("MA3Period", true));
				col.Remove(col.Find("MA3PriceType", true));
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
				col.Remove(col.Find("MA3RisingColor", true));
				col.Remove(col.Find("MA3RisingColorOpacity", true));
				col.Remove(col.Find("MA3FallingColor", true));
				col.Remove(col.Find("MA3FallingColorOpacity", true));
				col.Remove(col.Find("MA3LineWidth", true));
				col.Remove(col.Find("MA3LineDashStyle", true));
				col.Remove(col.Find("MA3LinePlotStyle", true));
			}
			
			if(_MA3Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("SuperSmootherFilterPoles3", true));
			}
			else if(_MA3Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase3", true));
				col.Remove(col.Find("T3tCount3", true));
				col.Remove(col.Find("T3vFactor3", true));
				col.Remove(col.Find("ButterWorthFilterPoles3", true));
				col.Remove(col.Find("GaussianFilterPoles3", true));
			}
			#endregion
			
			#region MA4
			if(!UseMA4)
			{
				col.Remove(col.Find("MA4Type", true));
				col.Remove(col.Find("MA4Period", true));
				col.Remove(col.Find("MA4PriceType", true));
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T3tCount4", true));
				col.Remove(col.Find("T3vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
				col.Remove(col.Find("MA4RisingColor", true));
				col.Remove(col.Find("MA4RisingColorOpacity", true));
				col.Remove(col.Find("MA4FallingColor", true));
				col.Remove(col.Find("MA4FallingColorOpacity", true));
				col.Remove(col.Find("MA4LineWidth", true));
				col.Remove(col.Find("MA4LineDashStyle", true));
				col.Remove(col.Find("MA4LinePlotStyle", true));
			}
			
			if(_MA4Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("SuperSmootherFilterPoles4", true));
			}
			else if(_MA4Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase4", true));
				col.Remove(col.Find("T4tCount4", true));
				col.Remove(col.Find("T4vFactor4", true));
				col.Remove(col.Find("ButterWorthFilterPoles4", true));
				col.Remove(col.Find("GaussianFilterPoles4", true));
			}
			#endregion
			
			#region MA5
			if(!UseMA5)
			{
				col.Remove(col.Find("MA5Type", true));
				col.Remove(col.Find("MA5Period", true));
				col.Remove(col.Find("MA5PriceType", true));
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T3tCount5", true));
				col.Remove(col.Find("T3vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
				col.Remove(col.Find("MA5RisingColor", true));
				col.Remove(col.Find("MA5RisingColorOpacity", true));
				col.Remove(col.Find("MA5FallingColor", true));
				col.Remove(col.Find("MA5FallingColorOpacity", true));
				col.Remove(col.Find("MA5LineWidth", true));
				col.Remove(col.Find("MA5LineDashStyle", true));
				col.Remove(col.Find("MA5LinePlotStyle", true));
			}
			
			if(_MA5Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("SuperSmootherFilterPoles5", true));
			}
			else if(_MA5Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase5", true));
				col.Remove(col.Find("T5tCount5", true));
				col.Remove(col.Find("T5vFactor5", true));
				col.Remove(col.Find("ButterWorthFilterPoles5", true));
				col.Remove(col.Find("GaussianFilterPoles5", true));
			}
			#endregion
			
			#region MA6
			if(!UseMA6)
			{
				col.Remove(col.Find("MA6Type", true));
				col.Remove(col.Find("MA6Period", true));
				col.Remove(col.Find("MA6PriceType", true));
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T3tCount6", true));
				col.Remove(col.Find("T3vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
				col.Remove(col.Find("MA6RisingColor", true));
				col.Remove(col.Find("MA6RisingColorOpacity", true));
				col.Remove(col.Find("MA6FallingColor", true));
				col.Remove(col.Find("MA6FallingColorOpacity", true));
				col.Remove(col.Find("MA6LineWidth", true));
				col.Remove(col.Find("MA6LineDashStyle", true));
				col.Remove(col.Find("MA6LinePlotStyle", true));
			}
			
			if(_MA6Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("SuperSmootherFilterPoles6", true));
			}
			else if(_MA6Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase6", true));
				col.Remove(col.Find("T6tCount6", true));
				col.Remove(col.Find("T6vFactor6", true));
				col.Remove(col.Find("ButterWorthFilterPoles6", true));
				col.Remove(col.Find("GaussianFilterPoles6", true));
			}
			#endregion
			
			#region MA7
			if(!UseMA7)
			{
				col.Remove(col.Find("MA7Type", true));
				col.Remove(col.Find("MA7Period", true));
				col.Remove(col.Find("MA7PriceType", true));
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T3tCount7", true));
				col.Remove(col.Find("T3vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
				col.Remove(col.Find("MA7RisingColor", true));
				col.Remove(col.Find("MA7RisingColorOpacity", true));
				col.Remove(col.Find("MA7FallingColor", true));
				col.Remove(col.Find("MA7FallingColorOpacity", true));
				col.Remove(col.Find("MA7LineWidth", true));
				col.Remove(col.Find("MA7LineDashStyle", true));
				col.Remove(col.Find("MA7LinePlotStyle", true));
			}
			
			if(_MA7Type == MAPackageMATypeV13.SMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.EMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.HMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.TMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.TEMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.T3)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.WMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.VWMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.ZLEMA)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.LinReg)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.JMA)
			{
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.LLMA)
			{
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.ButterWorthFilter)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.GaussianFilter)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("SuperSmootherFilterPoles7", true));
			}
			else if(_MA7Type == MAPackageMATypeV13.SuperSmootherFilter)
			{
				col.Remove(col.Find("JMAPhase7", true));
				col.Remove(col.Find("T7tCount7", true));
				col.Remove(col.Find("T7vFactor7", true));
				col.Remove(col.Find("ButterWorthFilterPoles7", true));
				col.Remove(col.Find("GaussianFilterPoles7", true));
			}
			#endregion
        }
        #endregion
		
		#region ICustomTypeDescriptor Members
        public AttributeCollection GetAttributes()
        {return TypeDescriptor.GetAttributes(GetType());}

        public string GetClassName()
        {return TypeDescriptor.GetClassName(GetType());}

        public string GetComponentName()
        {return TypeDescriptor.GetComponentName(GetType());}

        public TypeConverter GetConverter()
        {return TypeDescriptor.GetConverter(GetType());}

        public EventDescriptor GetDefaultEvent()
        {return TypeDescriptor.GetDefaultEvent(GetType());}

        public PropertyDescriptor GetDefaultProperty()
        {return TypeDescriptor.GetDefaultProperty(GetType());}

        public object GetEditor(Type editorBaseType)
        {return TypeDescriptor.GetEditor(GetType(), editorBaseType);}

        public EventDescriptorCollection GetEvents(Attribute[] attributes)
        {return TypeDescriptor.GetEvents(GetType(), attributes);}

        public EventDescriptorCollection GetEvents()
        {return TypeDescriptor.GetEvents(GetType());}

        public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {	
			PropertyDescriptorCollection orig = TypeDescriptor.GetProperties(GetType(), attributes);
            PropertyDescriptor[] arr = new PropertyDescriptor[orig.Count];
            orig.CopyTo(arr, 0);
            PropertyDescriptorCollection col = new PropertyDescriptorCollection(arr);
            ModifyProperties(col);
            return col;
		}

        public PropertyDescriptorCollection GetProperties()
        {return TypeDescriptor.GetProperties(GetType());}

        public object GetPropertyOwner(PropertyDescriptor pd)
        {return this;}
        #endregion
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MAPackage[] cacheMAPackage = null;

        private static MAPackage checkMAPackage = new MAPackage();

        /// <summary>
        /// MAPackage V1.3
        /// </summary>
        /// <returns></returns>
        public MAPackage MAPackage(int butterWorthFilterPoles1, int butterWorthFilterPoles2, int butterWorthFilterPoles3, int butterWorthFilterPoles4, int butterWorthFilterPoles5, int butterWorthFilterPoles6, int butterWorthFilterPoles7, int gaussianFilterPoles1, int gaussianFilterPoles2, int gaussianFilterPoles3, int gaussianFilterPoles4, int gaussianFilterPoles5, int gaussianFilterPoles6, int gaussianFilterPoles7, int jMAPhase1, int jMAPhase2, int jMAPhase3, int jMAPhase4, int jMAPhase5, int jMAPhase6, int jMAPhase7, Color mA1FallingColor, int mA1FallingColorOpacity, MAPackageDashStyleV13 mA1LineDashStyle, MAPackagePlotStyleV13 mA1LinePlotStyle, int mA1LineWidth, int mA1Period, MAPackageMAPriceTypesV13 mA1PriceType, Color mA1RisingColor, int mA1RisingColorOpacity, MAPackageMATypeV13 mA1Type, Color mA2FallingColor, int mA2FallingColorOpacity, MAPackageDashStyleV13 mA2LineDashStyle, MAPackagePlotStyleV13 mA2LinePlotStyle, int mA2LineWidth, int mA2Period, MAPackageMAPriceTypesV13 mA2PriceType, Color mA2RisingColor, int mA2RisingColorOpacity, MAPackageMATypeV13 mA2Type, Color mA3FallingColor, int mA3FallingColorOpacity, MAPackageDashStyleV13 mA3LineDashStyle, MAPackagePlotStyleV13 mA3LinePlotStyle, int mA3LineWidth, int mA3Period, MAPackageMAPriceTypesV13 mA3PriceType, Color mA3RisingColor, int mA3RisingColorOpacity, MAPackageMATypeV13 mA3Type, Color mA4FallingColor, int mA4FallingColorOpacity, MAPackageDashStyleV13 mA4LineDashStyle, MAPackagePlotStyleV13 mA4LinePlotStyle, int mA4LineWidth, int mA4Period, MAPackageMAPriceTypesV13 mA4PriceType, Color mA4RisingColor, int mA4RisingColorOpacity, MAPackageMATypeV13 mA4Type, Color mA5FallingColor, int mA5FallingColorOpacity, MAPackageDashStyleV13 mA5LineDashStyle, MAPackagePlotStyleV13 mA5LinePlotStyle, int mA5LineWidth, int mA5Period, MAPackageMAPriceTypesV13 mA5PriceType, Color mA5RisingColor, int mA5RisingColorOpacity, MAPackageMATypeV13 mA5Type, Color mA6FallingColor, int mA6FallingColorOpacity, MAPackageDashStyleV13 mA6LineDashStyle, MAPackagePlotStyleV13 mA6LinePlotStyle, int mA6LineWidth, int mA6Period, MAPackageMAPriceTypesV13 mA6PriceType, Color mA6RisingColor, int mA6RisingColorOpacity, MAPackageMATypeV13 mA6Type, Color mA7FallingColor, int mA7FallingColorOpacity, MAPackageDashStyleV13 mA7LineDashStyle, MAPackagePlotStyleV13 mA7LinePlotStyle, int mA7LineWidth, int mA7Period, MAPackageMAPriceTypesV13 mA7PriceType, Color mA7RisingColor, int mA7RisingColorOpacity, MAPackageMATypeV13 mA7Type, int superSmootherFilterPoles1, int superSmootherFilterPoles2, int superSmootherFilterPoles3, int superSmootherFilterPoles4, int superSmootherFilterPoles5, int superSmootherFilterPoles6, int superSmootherFilterPoles7, int t3tCount1, int t3tCount2, int t3tCount3, int t3tCount4, int t3tCount5, int t3tCount6, int t3tCount7, double t3vFactor1, double t3vFactor2, double t3vFactor3, double t3vFactor4, double t3vFactor5, double t3vFactor6, double t3vFactor7, bool useMA1, bool useMA2, bool useMA3, bool useMA4, bool useMA5, bool useMA6, bool useMA7)
        {
            return MAPackage(Input, butterWorthFilterPoles1, butterWorthFilterPoles2, butterWorthFilterPoles3, butterWorthFilterPoles4, butterWorthFilterPoles5, butterWorthFilterPoles6, butterWorthFilterPoles7, gaussianFilterPoles1, gaussianFilterPoles2, gaussianFilterPoles3, gaussianFilterPoles4, gaussianFilterPoles5, gaussianFilterPoles6, gaussianFilterPoles7, jMAPhase1, jMAPhase2, jMAPhase3, jMAPhase4, jMAPhase5, jMAPhase6, jMAPhase7, mA1FallingColor, mA1FallingColorOpacity, mA1LineDashStyle, mA1LinePlotStyle, mA1LineWidth, mA1Period, mA1PriceType, mA1RisingColor, mA1RisingColorOpacity, mA1Type, mA2FallingColor, mA2FallingColorOpacity, mA2LineDashStyle, mA2LinePlotStyle, mA2LineWidth, mA2Period, mA2PriceType, mA2RisingColor, mA2RisingColorOpacity, mA2Type, mA3FallingColor, mA3FallingColorOpacity, mA3LineDashStyle, mA3LinePlotStyle, mA3LineWidth, mA3Period, mA3PriceType, mA3RisingColor, mA3RisingColorOpacity, mA3Type, mA4FallingColor, mA4FallingColorOpacity, mA4LineDashStyle, mA4LinePlotStyle, mA4LineWidth, mA4Period, mA4PriceType, mA4RisingColor, mA4RisingColorOpacity, mA4Type, mA5FallingColor, mA5FallingColorOpacity, mA5LineDashStyle, mA5LinePlotStyle, mA5LineWidth, mA5Period, mA5PriceType, mA5RisingColor, mA5RisingColorOpacity, mA5Type, mA6FallingColor, mA6FallingColorOpacity, mA6LineDashStyle, mA6LinePlotStyle, mA6LineWidth, mA6Period, mA6PriceType, mA6RisingColor, mA6RisingColorOpacity, mA6Type, mA7FallingColor, mA7FallingColorOpacity, mA7LineDashStyle, mA7LinePlotStyle, mA7LineWidth, mA7Period, mA7PriceType, mA7RisingColor, mA7RisingColorOpacity, mA7Type, superSmootherFilterPoles1, superSmootherFilterPoles2, superSmootherFilterPoles3, superSmootherFilterPoles4, superSmootherFilterPoles5, superSmootherFilterPoles6, superSmootherFilterPoles7, t3tCount1, t3tCount2, t3tCount3, t3tCount4, t3tCount5, t3tCount6, t3tCount7, t3vFactor1, t3vFactor2, t3vFactor3, t3vFactor4, t3vFactor5, t3vFactor6, t3vFactor7, useMA1, useMA2, useMA3, useMA4, useMA5, useMA6, useMA7);
        }

        /// <summary>
        /// MAPackage V1.3
        /// </summary>
        /// <returns></returns>
        public MAPackage MAPackage(Data.IDataSeries input, int butterWorthFilterPoles1, int butterWorthFilterPoles2, int butterWorthFilterPoles3, int butterWorthFilterPoles4, int butterWorthFilterPoles5, int butterWorthFilterPoles6, int butterWorthFilterPoles7, int gaussianFilterPoles1, int gaussianFilterPoles2, int gaussianFilterPoles3, int gaussianFilterPoles4, int gaussianFilterPoles5, int gaussianFilterPoles6, int gaussianFilterPoles7, int jMAPhase1, int jMAPhase2, int jMAPhase3, int jMAPhase4, int jMAPhase5, int jMAPhase6, int jMAPhase7, Color mA1FallingColor, int mA1FallingColorOpacity, MAPackageDashStyleV13 mA1LineDashStyle, MAPackagePlotStyleV13 mA1LinePlotStyle, int mA1LineWidth, int mA1Period, MAPackageMAPriceTypesV13 mA1PriceType, Color mA1RisingColor, int mA1RisingColorOpacity, MAPackageMATypeV13 mA1Type, Color mA2FallingColor, int mA2FallingColorOpacity, MAPackageDashStyleV13 mA2LineDashStyle, MAPackagePlotStyleV13 mA2LinePlotStyle, int mA2LineWidth, int mA2Period, MAPackageMAPriceTypesV13 mA2PriceType, Color mA2RisingColor, int mA2RisingColorOpacity, MAPackageMATypeV13 mA2Type, Color mA3FallingColor, int mA3FallingColorOpacity, MAPackageDashStyleV13 mA3LineDashStyle, MAPackagePlotStyleV13 mA3LinePlotStyle, int mA3LineWidth, int mA3Period, MAPackageMAPriceTypesV13 mA3PriceType, Color mA3RisingColor, int mA3RisingColorOpacity, MAPackageMATypeV13 mA3Type, Color mA4FallingColor, int mA4FallingColorOpacity, MAPackageDashStyleV13 mA4LineDashStyle, MAPackagePlotStyleV13 mA4LinePlotStyle, int mA4LineWidth, int mA4Period, MAPackageMAPriceTypesV13 mA4PriceType, Color mA4RisingColor, int mA4RisingColorOpacity, MAPackageMATypeV13 mA4Type, Color mA5FallingColor, int mA5FallingColorOpacity, MAPackageDashStyleV13 mA5LineDashStyle, MAPackagePlotStyleV13 mA5LinePlotStyle, int mA5LineWidth, int mA5Period, MAPackageMAPriceTypesV13 mA5PriceType, Color mA5RisingColor, int mA5RisingColorOpacity, MAPackageMATypeV13 mA5Type, Color mA6FallingColor, int mA6FallingColorOpacity, MAPackageDashStyleV13 mA6LineDashStyle, MAPackagePlotStyleV13 mA6LinePlotStyle, int mA6LineWidth, int mA6Period, MAPackageMAPriceTypesV13 mA6PriceType, Color mA6RisingColor, int mA6RisingColorOpacity, MAPackageMATypeV13 mA6Type, Color mA7FallingColor, int mA7FallingColorOpacity, MAPackageDashStyleV13 mA7LineDashStyle, MAPackagePlotStyleV13 mA7LinePlotStyle, int mA7LineWidth, int mA7Period, MAPackageMAPriceTypesV13 mA7PriceType, Color mA7RisingColor, int mA7RisingColorOpacity, MAPackageMATypeV13 mA7Type, int superSmootherFilterPoles1, int superSmootherFilterPoles2, int superSmootherFilterPoles3, int superSmootherFilterPoles4, int superSmootherFilterPoles5, int superSmootherFilterPoles6, int superSmootherFilterPoles7, int t3tCount1, int t3tCount2, int t3tCount3, int t3tCount4, int t3tCount5, int t3tCount6, int t3tCount7, double t3vFactor1, double t3vFactor2, double t3vFactor3, double t3vFactor4, double t3vFactor5, double t3vFactor6, double t3vFactor7, bool useMA1, bool useMA2, bool useMA3, bool useMA4, bool useMA5, bool useMA6, bool useMA7)
        {
            if (cacheMAPackage != null)
                for (int idx = 0; idx < cacheMAPackage.Length; idx++)
                    if (cacheMAPackage[idx].ButterWorthFilterPoles1 == butterWorthFilterPoles1 && cacheMAPackage[idx].ButterWorthFilterPoles2 == butterWorthFilterPoles2 && cacheMAPackage[idx].ButterWorthFilterPoles3 == butterWorthFilterPoles3 && cacheMAPackage[idx].ButterWorthFilterPoles4 == butterWorthFilterPoles4 && cacheMAPackage[idx].ButterWorthFilterPoles5 == butterWorthFilterPoles5 && cacheMAPackage[idx].ButterWorthFilterPoles6 == butterWorthFilterPoles6 && cacheMAPackage[idx].ButterWorthFilterPoles7 == butterWorthFilterPoles7 && cacheMAPackage[idx].GaussianFilterPoles1 == gaussianFilterPoles1 && cacheMAPackage[idx].GaussianFilterPoles2 == gaussianFilterPoles2 && cacheMAPackage[idx].GaussianFilterPoles3 == gaussianFilterPoles3 && cacheMAPackage[idx].GaussianFilterPoles4 == gaussianFilterPoles4 && cacheMAPackage[idx].GaussianFilterPoles5 == gaussianFilterPoles5 && cacheMAPackage[idx].GaussianFilterPoles6 == gaussianFilterPoles6 && cacheMAPackage[idx].GaussianFilterPoles7 == gaussianFilterPoles7 && cacheMAPackage[idx].JMAPhase1 == jMAPhase1 && cacheMAPackage[idx].JMAPhase2 == jMAPhase2 && cacheMAPackage[idx].JMAPhase3 == jMAPhase3 && cacheMAPackage[idx].JMAPhase4 == jMAPhase4 && cacheMAPackage[idx].JMAPhase5 == jMAPhase5 && cacheMAPackage[idx].JMAPhase6 == jMAPhase6 && cacheMAPackage[idx].JMAPhase7 == jMAPhase7 && cacheMAPackage[idx].MA1FallingColor == mA1FallingColor && cacheMAPackage[idx].MA1FallingColorOpacity == mA1FallingColorOpacity && cacheMAPackage[idx].MA1LineDashStyle == mA1LineDashStyle && cacheMAPackage[idx].MA1LinePlotStyle == mA1LinePlotStyle && cacheMAPackage[idx].MA1LineWidth == mA1LineWidth && cacheMAPackage[idx].MA1Period == mA1Period && cacheMAPackage[idx].MA1PriceType == mA1PriceType && cacheMAPackage[idx].MA1RisingColor == mA1RisingColor && cacheMAPackage[idx].MA1RisingColorOpacity == mA1RisingColorOpacity && cacheMAPackage[idx].MA1Type == mA1Type && cacheMAPackage[idx].MA2FallingColor == mA2FallingColor && cacheMAPackage[idx].MA2FallingColorOpacity == mA2FallingColorOpacity && cacheMAPackage[idx].MA2LineDashStyle == mA2LineDashStyle && cacheMAPackage[idx].MA2LinePlotStyle == mA2LinePlotStyle && cacheMAPackage[idx].MA2LineWidth == mA2LineWidth && cacheMAPackage[idx].MA2Period == mA2Period && cacheMAPackage[idx].MA2PriceType == mA2PriceType && cacheMAPackage[idx].MA2RisingColor == mA2RisingColor && cacheMAPackage[idx].MA2RisingColorOpacity == mA2RisingColorOpacity && cacheMAPackage[idx].MA2Type == mA2Type && cacheMAPackage[idx].MA3FallingColor == mA3FallingColor && cacheMAPackage[idx].MA3FallingColorOpacity == mA3FallingColorOpacity && cacheMAPackage[idx].MA3LineDashStyle == mA3LineDashStyle && cacheMAPackage[idx].MA3LinePlotStyle == mA3LinePlotStyle && cacheMAPackage[idx].MA3LineWidth == mA3LineWidth && cacheMAPackage[idx].MA3Period == mA3Period && cacheMAPackage[idx].MA3PriceType == mA3PriceType && cacheMAPackage[idx].MA3RisingColor == mA3RisingColor && cacheMAPackage[idx].MA3RisingColorOpacity == mA3RisingColorOpacity && cacheMAPackage[idx].MA3Type == mA3Type && cacheMAPackage[idx].MA4FallingColor == mA4FallingColor && cacheMAPackage[idx].MA4FallingColorOpacity == mA4FallingColorOpacity && cacheMAPackage[idx].MA4LineDashStyle == mA4LineDashStyle && cacheMAPackage[idx].MA4LinePlotStyle == mA4LinePlotStyle && cacheMAPackage[idx].MA4LineWidth == mA4LineWidth && cacheMAPackage[idx].MA4Period == mA4Period && cacheMAPackage[idx].MA4PriceType == mA4PriceType && cacheMAPackage[idx].MA4RisingColor == mA4RisingColor && cacheMAPackage[idx].MA4RisingColorOpacity == mA4RisingColorOpacity && cacheMAPackage[idx].MA4Type == mA4Type && cacheMAPackage[idx].MA5FallingColor == mA5FallingColor && cacheMAPackage[idx].MA5FallingColorOpacity == mA5FallingColorOpacity && cacheMAPackage[idx].MA5LineDashStyle == mA5LineDashStyle && cacheMAPackage[idx].MA5LinePlotStyle == mA5LinePlotStyle && cacheMAPackage[idx].MA5LineWidth == mA5LineWidth && cacheMAPackage[idx].MA5Period == mA5Period && cacheMAPackage[idx].MA5PriceType == mA5PriceType && cacheMAPackage[idx].MA5RisingColor == mA5RisingColor && cacheMAPackage[idx].MA5RisingColorOpacity == mA5RisingColorOpacity && cacheMAPackage[idx].MA5Type == mA5Type && cacheMAPackage[idx].MA6FallingColor == mA6FallingColor && cacheMAPackage[idx].MA6FallingColorOpacity == mA6FallingColorOpacity && cacheMAPackage[idx].MA6LineDashStyle == mA6LineDashStyle && cacheMAPackage[idx].MA6LinePlotStyle == mA6LinePlotStyle && cacheMAPackage[idx].MA6LineWidth == mA6LineWidth && cacheMAPackage[idx].MA6Period == mA6Period && cacheMAPackage[idx].MA6PriceType == mA6PriceType && cacheMAPackage[idx].MA6RisingColor == mA6RisingColor && cacheMAPackage[idx].MA6RisingColorOpacity == mA6RisingColorOpacity && cacheMAPackage[idx].MA6Type == mA6Type && cacheMAPackage[idx].MA7FallingColor == mA7FallingColor && cacheMAPackage[idx].MA7FallingColorOpacity == mA7FallingColorOpacity && cacheMAPackage[idx].MA7LineDashStyle == mA7LineDashStyle && cacheMAPackage[idx].MA7LinePlotStyle == mA7LinePlotStyle && cacheMAPackage[idx].MA7LineWidth == mA7LineWidth && cacheMAPackage[idx].MA7Period == mA7Period && cacheMAPackage[idx].MA7PriceType == mA7PriceType && cacheMAPackage[idx].MA7RisingColor == mA7RisingColor && cacheMAPackage[idx].MA7RisingColorOpacity == mA7RisingColorOpacity && cacheMAPackage[idx].MA7Type == mA7Type && cacheMAPackage[idx].SuperSmootherFilterPoles1 == superSmootherFilterPoles1 && cacheMAPackage[idx].SuperSmootherFilterPoles2 == superSmootherFilterPoles2 && cacheMAPackage[idx].SuperSmootherFilterPoles3 == superSmootherFilterPoles3 && cacheMAPackage[idx].SuperSmootherFilterPoles4 == superSmootherFilterPoles4 && cacheMAPackage[idx].SuperSmootherFilterPoles5 == superSmootherFilterPoles5 && cacheMAPackage[idx].SuperSmootherFilterPoles6 == superSmootherFilterPoles6 && cacheMAPackage[idx].SuperSmootherFilterPoles7 == superSmootherFilterPoles7 && cacheMAPackage[idx].T3tCount1 == t3tCount1 && cacheMAPackage[idx].T3tCount2 == t3tCount2 && cacheMAPackage[idx].T3tCount3 == t3tCount3 && cacheMAPackage[idx].T3tCount4 == t3tCount4 && cacheMAPackage[idx].T3tCount5 == t3tCount5 && cacheMAPackage[idx].T3tCount6 == t3tCount6 && cacheMAPackage[idx].T3tCount7 == t3tCount7 && Math.Abs(cacheMAPackage[idx].T3vFactor1 - t3vFactor1) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor2 - t3vFactor2) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor3 - t3vFactor3) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor4 - t3vFactor4) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor5 - t3vFactor5) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor6 - t3vFactor6) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor7 - t3vFactor7) <= double.Epsilon && cacheMAPackage[idx].UseMA1 == useMA1 && cacheMAPackage[idx].UseMA2 == useMA2 && cacheMAPackage[idx].UseMA3 == useMA3 && cacheMAPackage[idx].UseMA4 == useMA4 && cacheMAPackage[idx].UseMA5 == useMA5 && cacheMAPackage[idx].UseMA6 == useMA6 && cacheMAPackage[idx].UseMA7 == useMA7 && cacheMAPackage[idx].EqualsInput(input))
                        return cacheMAPackage[idx];

            lock (checkMAPackage)
            {
                checkMAPackage.ButterWorthFilterPoles1 = butterWorthFilterPoles1;
                butterWorthFilterPoles1 = checkMAPackage.ButterWorthFilterPoles1;
                checkMAPackage.ButterWorthFilterPoles2 = butterWorthFilterPoles2;
                butterWorthFilterPoles2 = checkMAPackage.ButterWorthFilterPoles2;
                checkMAPackage.ButterWorthFilterPoles3 = butterWorthFilterPoles3;
                butterWorthFilterPoles3 = checkMAPackage.ButterWorthFilterPoles3;
                checkMAPackage.ButterWorthFilterPoles4 = butterWorthFilterPoles4;
                butterWorthFilterPoles4 = checkMAPackage.ButterWorthFilterPoles4;
                checkMAPackage.ButterWorthFilterPoles5 = butterWorthFilterPoles5;
                butterWorthFilterPoles5 = checkMAPackage.ButterWorthFilterPoles5;
                checkMAPackage.ButterWorthFilterPoles6 = butterWorthFilterPoles6;
                butterWorthFilterPoles6 = checkMAPackage.ButterWorthFilterPoles6;
                checkMAPackage.ButterWorthFilterPoles7 = butterWorthFilterPoles7;
                butterWorthFilterPoles7 = checkMAPackage.ButterWorthFilterPoles7;
                checkMAPackage.GaussianFilterPoles1 = gaussianFilterPoles1;
                gaussianFilterPoles1 = checkMAPackage.GaussianFilterPoles1;
                checkMAPackage.GaussianFilterPoles2 = gaussianFilterPoles2;
                gaussianFilterPoles2 = checkMAPackage.GaussianFilterPoles2;
                checkMAPackage.GaussianFilterPoles3 = gaussianFilterPoles3;
                gaussianFilterPoles3 = checkMAPackage.GaussianFilterPoles3;
                checkMAPackage.GaussianFilterPoles4 = gaussianFilterPoles4;
                gaussianFilterPoles4 = checkMAPackage.GaussianFilterPoles4;
                checkMAPackage.GaussianFilterPoles5 = gaussianFilterPoles5;
                gaussianFilterPoles5 = checkMAPackage.GaussianFilterPoles5;
                checkMAPackage.GaussianFilterPoles6 = gaussianFilterPoles6;
                gaussianFilterPoles6 = checkMAPackage.GaussianFilterPoles6;
                checkMAPackage.GaussianFilterPoles7 = gaussianFilterPoles7;
                gaussianFilterPoles7 = checkMAPackage.GaussianFilterPoles7;
                checkMAPackage.JMAPhase1 = jMAPhase1;
                jMAPhase1 = checkMAPackage.JMAPhase1;
                checkMAPackage.JMAPhase2 = jMAPhase2;
                jMAPhase2 = checkMAPackage.JMAPhase2;
                checkMAPackage.JMAPhase3 = jMAPhase3;
                jMAPhase3 = checkMAPackage.JMAPhase3;
                checkMAPackage.JMAPhase4 = jMAPhase4;
                jMAPhase4 = checkMAPackage.JMAPhase4;
                checkMAPackage.JMAPhase5 = jMAPhase5;
                jMAPhase5 = checkMAPackage.JMAPhase5;
                checkMAPackage.JMAPhase6 = jMAPhase6;
                jMAPhase6 = checkMAPackage.JMAPhase6;
                checkMAPackage.JMAPhase7 = jMAPhase7;
                jMAPhase7 = checkMAPackage.JMAPhase7;
                checkMAPackage.MA1FallingColor = mA1FallingColor;
                mA1FallingColor = checkMAPackage.MA1FallingColor;
                checkMAPackage.MA1FallingColorOpacity = mA1FallingColorOpacity;
                mA1FallingColorOpacity = checkMAPackage.MA1FallingColorOpacity;
                checkMAPackage.MA1LineDashStyle = mA1LineDashStyle;
                mA1LineDashStyle = checkMAPackage.MA1LineDashStyle;
                checkMAPackage.MA1LinePlotStyle = mA1LinePlotStyle;
                mA1LinePlotStyle = checkMAPackage.MA1LinePlotStyle;
                checkMAPackage.MA1LineWidth = mA1LineWidth;
                mA1LineWidth = checkMAPackage.MA1LineWidth;
                checkMAPackage.MA1Period = mA1Period;
                mA1Period = checkMAPackage.MA1Period;
                checkMAPackage.MA1PriceType = mA1PriceType;
                mA1PriceType = checkMAPackage.MA1PriceType;
                checkMAPackage.MA1RisingColor = mA1RisingColor;
                mA1RisingColor = checkMAPackage.MA1RisingColor;
                checkMAPackage.MA1RisingColorOpacity = mA1RisingColorOpacity;
                mA1RisingColorOpacity = checkMAPackage.MA1RisingColorOpacity;
                checkMAPackage.MA1Type = mA1Type;
                mA1Type = checkMAPackage.MA1Type;
                checkMAPackage.MA2FallingColor = mA2FallingColor;
                mA2FallingColor = checkMAPackage.MA2FallingColor;
                checkMAPackage.MA2FallingColorOpacity = mA2FallingColorOpacity;
                mA2FallingColorOpacity = checkMAPackage.MA2FallingColorOpacity;
                checkMAPackage.MA2LineDashStyle = mA2LineDashStyle;
                mA2LineDashStyle = checkMAPackage.MA2LineDashStyle;
                checkMAPackage.MA2LinePlotStyle = mA2LinePlotStyle;
                mA2LinePlotStyle = checkMAPackage.MA2LinePlotStyle;
                checkMAPackage.MA2LineWidth = mA2LineWidth;
                mA2LineWidth = checkMAPackage.MA2LineWidth;
                checkMAPackage.MA2Period = mA2Period;
                mA2Period = checkMAPackage.MA2Period;
                checkMAPackage.MA2PriceType = mA2PriceType;
                mA2PriceType = checkMAPackage.MA2PriceType;
                checkMAPackage.MA2RisingColor = mA2RisingColor;
                mA2RisingColor = checkMAPackage.MA2RisingColor;
                checkMAPackage.MA2RisingColorOpacity = mA2RisingColorOpacity;
                mA2RisingColorOpacity = checkMAPackage.MA2RisingColorOpacity;
                checkMAPackage.MA2Type = mA2Type;
                mA2Type = checkMAPackage.MA2Type;
                checkMAPackage.MA3FallingColor = mA3FallingColor;
                mA3FallingColor = checkMAPackage.MA3FallingColor;
                checkMAPackage.MA3FallingColorOpacity = mA3FallingColorOpacity;
                mA3FallingColorOpacity = checkMAPackage.MA3FallingColorOpacity;
                checkMAPackage.MA3LineDashStyle = mA3LineDashStyle;
                mA3LineDashStyle = checkMAPackage.MA3LineDashStyle;
                checkMAPackage.MA3LinePlotStyle = mA3LinePlotStyle;
                mA3LinePlotStyle = checkMAPackage.MA3LinePlotStyle;
                checkMAPackage.MA3LineWidth = mA3LineWidth;
                mA3LineWidth = checkMAPackage.MA3LineWidth;
                checkMAPackage.MA3Period = mA3Period;
                mA3Period = checkMAPackage.MA3Period;
                checkMAPackage.MA3PriceType = mA3PriceType;
                mA3PriceType = checkMAPackage.MA3PriceType;
                checkMAPackage.MA3RisingColor = mA3RisingColor;
                mA3RisingColor = checkMAPackage.MA3RisingColor;
                checkMAPackage.MA3RisingColorOpacity = mA3RisingColorOpacity;
                mA3RisingColorOpacity = checkMAPackage.MA3RisingColorOpacity;
                checkMAPackage.MA3Type = mA3Type;
                mA3Type = checkMAPackage.MA3Type;
                checkMAPackage.MA4FallingColor = mA4FallingColor;
                mA4FallingColor = checkMAPackage.MA4FallingColor;
                checkMAPackage.MA4FallingColorOpacity = mA4FallingColorOpacity;
                mA4FallingColorOpacity = checkMAPackage.MA4FallingColorOpacity;
                checkMAPackage.MA4LineDashStyle = mA4LineDashStyle;
                mA4LineDashStyle = checkMAPackage.MA4LineDashStyle;
                checkMAPackage.MA4LinePlotStyle = mA4LinePlotStyle;
                mA4LinePlotStyle = checkMAPackage.MA4LinePlotStyle;
                checkMAPackage.MA4LineWidth = mA4LineWidth;
                mA4LineWidth = checkMAPackage.MA4LineWidth;
                checkMAPackage.MA4Period = mA4Period;
                mA4Period = checkMAPackage.MA4Period;
                checkMAPackage.MA4PriceType = mA4PriceType;
                mA4PriceType = checkMAPackage.MA4PriceType;
                checkMAPackage.MA4RisingColor = mA4RisingColor;
                mA4RisingColor = checkMAPackage.MA4RisingColor;
                checkMAPackage.MA4RisingColorOpacity = mA4RisingColorOpacity;
                mA4RisingColorOpacity = checkMAPackage.MA4RisingColorOpacity;
                checkMAPackage.MA4Type = mA4Type;
                mA4Type = checkMAPackage.MA4Type;
                checkMAPackage.MA5FallingColor = mA5FallingColor;
                mA5FallingColor = checkMAPackage.MA5FallingColor;
                checkMAPackage.MA5FallingColorOpacity = mA5FallingColorOpacity;
                mA5FallingColorOpacity = checkMAPackage.MA5FallingColorOpacity;
                checkMAPackage.MA5LineDashStyle = mA5LineDashStyle;
                mA5LineDashStyle = checkMAPackage.MA5LineDashStyle;
                checkMAPackage.MA5LinePlotStyle = mA5LinePlotStyle;
                mA5LinePlotStyle = checkMAPackage.MA5LinePlotStyle;
                checkMAPackage.MA5LineWidth = mA5LineWidth;
                mA5LineWidth = checkMAPackage.MA5LineWidth;
                checkMAPackage.MA5Period = mA5Period;
                mA5Period = checkMAPackage.MA5Period;
                checkMAPackage.MA5PriceType = mA5PriceType;
                mA5PriceType = checkMAPackage.MA5PriceType;
                checkMAPackage.MA5RisingColor = mA5RisingColor;
                mA5RisingColor = checkMAPackage.MA5RisingColor;
                checkMAPackage.MA5RisingColorOpacity = mA5RisingColorOpacity;
                mA5RisingColorOpacity = checkMAPackage.MA5RisingColorOpacity;
                checkMAPackage.MA5Type = mA5Type;
                mA5Type = checkMAPackage.MA5Type;
                checkMAPackage.MA6FallingColor = mA6FallingColor;
                mA6FallingColor = checkMAPackage.MA6FallingColor;
                checkMAPackage.MA6FallingColorOpacity = mA6FallingColorOpacity;
                mA6FallingColorOpacity = checkMAPackage.MA6FallingColorOpacity;
                checkMAPackage.MA6LineDashStyle = mA6LineDashStyle;
                mA6LineDashStyle = checkMAPackage.MA6LineDashStyle;
                checkMAPackage.MA6LinePlotStyle = mA6LinePlotStyle;
                mA6LinePlotStyle = checkMAPackage.MA6LinePlotStyle;
                checkMAPackage.MA6LineWidth = mA6LineWidth;
                mA6LineWidth = checkMAPackage.MA6LineWidth;
                checkMAPackage.MA6Period = mA6Period;
                mA6Period = checkMAPackage.MA6Period;
                checkMAPackage.MA6PriceType = mA6PriceType;
                mA6PriceType = checkMAPackage.MA6PriceType;
                checkMAPackage.MA6RisingColor = mA6RisingColor;
                mA6RisingColor = checkMAPackage.MA6RisingColor;
                checkMAPackage.MA6RisingColorOpacity = mA6RisingColorOpacity;
                mA6RisingColorOpacity = checkMAPackage.MA6RisingColorOpacity;
                checkMAPackage.MA6Type = mA6Type;
                mA6Type = checkMAPackage.MA6Type;
                checkMAPackage.MA7FallingColor = mA7FallingColor;
                mA7FallingColor = checkMAPackage.MA7FallingColor;
                checkMAPackage.MA7FallingColorOpacity = mA7FallingColorOpacity;
                mA7FallingColorOpacity = checkMAPackage.MA7FallingColorOpacity;
                checkMAPackage.MA7LineDashStyle = mA7LineDashStyle;
                mA7LineDashStyle = checkMAPackage.MA7LineDashStyle;
                checkMAPackage.MA7LinePlotStyle = mA7LinePlotStyle;
                mA7LinePlotStyle = checkMAPackage.MA7LinePlotStyle;
                checkMAPackage.MA7LineWidth = mA7LineWidth;
                mA7LineWidth = checkMAPackage.MA7LineWidth;
                checkMAPackage.MA7Period = mA7Period;
                mA7Period = checkMAPackage.MA7Period;
                checkMAPackage.MA7PriceType = mA7PriceType;
                mA7PriceType = checkMAPackage.MA7PriceType;
                checkMAPackage.MA7RisingColor = mA7RisingColor;
                mA7RisingColor = checkMAPackage.MA7RisingColor;
                checkMAPackage.MA7RisingColorOpacity = mA7RisingColorOpacity;
                mA7RisingColorOpacity = checkMAPackage.MA7RisingColorOpacity;
                checkMAPackage.MA7Type = mA7Type;
                mA7Type = checkMAPackage.MA7Type;
                checkMAPackage.SuperSmootherFilterPoles1 = superSmootherFilterPoles1;
                superSmootherFilterPoles1 = checkMAPackage.SuperSmootherFilterPoles1;
                checkMAPackage.SuperSmootherFilterPoles2 = superSmootherFilterPoles2;
                superSmootherFilterPoles2 = checkMAPackage.SuperSmootherFilterPoles2;
                checkMAPackage.SuperSmootherFilterPoles3 = superSmootherFilterPoles3;
                superSmootherFilterPoles3 = checkMAPackage.SuperSmootherFilterPoles3;
                checkMAPackage.SuperSmootherFilterPoles4 = superSmootherFilterPoles4;
                superSmootherFilterPoles4 = checkMAPackage.SuperSmootherFilterPoles4;
                checkMAPackage.SuperSmootherFilterPoles5 = superSmootherFilterPoles5;
                superSmootherFilterPoles5 = checkMAPackage.SuperSmootherFilterPoles5;
                checkMAPackage.SuperSmootherFilterPoles6 = superSmootherFilterPoles6;
                superSmootherFilterPoles6 = checkMAPackage.SuperSmootherFilterPoles6;
                checkMAPackage.SuperSmootherFilterPoles7 = superSmootherFilterPoles7;
                superSmootherFilterPoles7 = checkMAPackage.SuperSmootherFilterPoles7;
                checkMAPackage.T3tCount1 = t3tCount1;
                t3tCount1 = checkMAPackage.T3tCount1;
                checkMAPackage.T3tCount2 = t3tCount2;
                t3tCount2 = checkMAPackage.T3tCount2;
                checkMAPackage.T3tCount3 = t3tCount3;
                t3tCount3 = checkMAPackage.T3tCount3;
                checkMAPackage.T3tCount4 = t3tCount4;
                t3tCount4 = checkMAPackage.T3tCount4;
                checkMAPackage.T3tCount5 = t3tCount5;
                t3tCount5 = checkMAPackage.T3tCount5;
                checkMAPackage.T3tCount6 = t3tCount6;
                t3tCount6 = checkMAPackage.T3tCount6;
                checkMAPackage.T3tCount7 = t3tCount7;
                t3tCount7 = checkMAPackage.T3tCount7;
                checkMAPackage.T3vFactor1 = t3vFactor1;
                t3vFactor1 = checkMAPackage.T3vFactor1;
                checkMAPackage.T3vFactor2 = t3vFactor2;
                t3vFactor2 = checkMAPackage.T3vFactor2;
                checkMAPackage.T3vFactor3 = t3vFactor3;
                t3vFactor3 = checkMAPackage.T3vFactor3;
                checkMAPackage.T3vFactor4 = t3vFactor4;
                t3vFactor4 = checkMAPackage.T3vFactor4;
                checkMAPackage.T3vFactor5 = t3vFactor5;
                t3vFactor5 = checkMAPackage.T3vFactor5;
                checkMAPackage.T3vFactor6 = t3vFactor6;
                t3vFactor6 = checkMAPackage.T3vFactor6;
                checkMAPackage.T3vFactor7 = t3vFactor7;
                t3vFactor7 = checkMAPackage.T3vFactor7;
                checkMAPackage.UseMA1 = useMA1;
                useMA1 = checkMAPackage.UseMA1;
                checkMAPackage.UseMA2 = useMA2;
                useMA2 = checkMAPackage.UseMA2;
                checkMAPackage.UseMA3 = useMA3;
                useMA3 = checkMAPackage.UseMA3;
                checkMAPackage.UseMA4 = useMA4;
                useMA4 = checkMAPackage.UseMA4;
                checkMAPackage.UseMA5 = useMA5;
                useMA5 = checkMAPackage.UseMA5;
                checkMAPackage.UseMA6 = useMA6;
                useMA6 = checkMAPackage.UseMA6;
                checkMAPackage.UseMA7 = useMA7;
                useMA7 = checkMAPackage.UseMA7;

                if (cacheMAPackage != null)
                    for (int idx = 0; idx < cacheMAPackage.Length; idx++)
                        if (cacheMAPackage[idx].ButterWorthFilterPoles1 == butterWorthFilterPoles1 && cacheMAPackage[idx].ButterWorthFilterPoles2 == butterWorthFilterPoles2 && cacheMAPackage[idx].ButterWorthFilterPoles3 == butterWorthFilterPoles3 && cacheMAPackage[idx].ButterWorthFilterPoles4 == butterWorthFilterPoles4 && cacheMAPackage[idx].ButterWorthFilterPoles5 == butterWorthFilterPoles5 && cacheMAPackage[idx].ButterWorthFilterPoles6 == butterWorthFilterPoles6 && cacheMAPackage[idx].ButterWorthFilterPoles7 == butterWorthFilterPoles7 && cacheMAPackage[idx].GaussianFilterPoles1 == gaussianFilterPoles1 && cacheMAPackage[idx].GaussianFilterPoles2 == gaussianFilterPoles2 && cacheMAPackage[idx].GaussianFilterPoles3 == gaussianFilterPoles3 && cacheMAPackage[idx].GaussianFilterPoles4 == gaussianFilterPoles4 && cacheMAPackage[idx].GaussianFilterPoles5 == gaussianFilterPoles5 && cacheMAPackage[idx].GaussianFilterPoles6 == gaussianFilterPoles6 && cacheMAPackage[idx].GaussianFilterPoles7 == gaussianFilterPoles7 && cacheMAPackage[idx].JMAPhase1 == jMAPhase1 && cacheMAPackage[idx].JMAPhase2 == jMAPhase2 && cacheMAPackage[idx].JMAPhase3 == jMAPhase3 && cacheMAPackage[idx].JMAPhase4 == jMAPhase4 && cacheMAPackage[idx].JMAPhase5 == jMAPhase5 && cacheMAPackage[idx].JMAPhase6 == jMAPhase6 && cacheMAPackage[idx].JMAPhase7 == jMAPhase7 && cacheMAPackage[idx].MA1FallingColor == mA1FallingColor && cacheMAPackage[idx].MA1FallingColorOpacity == mA1FallingColorOpacity && cacheMAPackage[idx].MA1LineDashStyle == mA1LineDashStyle && cacheMAPackage[idx].MA1LinePlotStyle == mA1LinePlotStyle && cacheMAPackage[idx].MA1LineWidth == mA1LineWidth && cacheMAPackage[idx].MA1Period == mA1Period && cacheMAPackage[idx].MA1PriceType == mA1PriceType && cacheMAPackage[idx].MA1RisingColor == mA1RisingColor && cacheMAPackage[idx].MA1RisingColorOpacity == mA1RisingColorOpacity && cacheMAPackage[idx].MA1Type == mA1Type && cacheMAPackage[idx].MA2FallingColor == mA2FallingColor && cacheMAPackage[idx].MA2FallingColorOpacity == mA2FallingColorOpacity && cacheMAPackage[idx].MA2LineDashStyle == mA2LineDashStyle && cacheMAPackage[idx].MA2LinePlotStyle == mA2LinePlotStyle && cacheMAPackage[idx].MA2LineWidth == mA2LineWidth && cacheMAPackage[idx].MA2Period == mA2Period && cacheMAPackage[idx].MA2PriceType == mA2PriceType && cacheMAPackage[idx].MA2RisingColor == mA2RisingColor && cacheMAPackage[idx].MA2RisingColorOpacity == mA2RisingColorOpacity && cacheMAPackage[idx].MA2Type == mA2Type && cacheMAPackage[idx].MA3FallingColor == mA3FallingColor && cacheMAPackage[idx].MA3FallingColorOpacity == mA3FallingColorOpacity && cacheMAPackage[idx].MA3LineDashStyle == mA3LineDashStyle && cacheMAPackage[idx].MA3LinePlotStyle == mA3LinePlotStyle && cacheMAPackage[idx].MA3LineWidth == mA3LineWidth && cacheMAPackage[idx].MA3Period == mA3Period && cacheMAPackage[idx].MA3PriceType == mA3PriceType && cacheMAPackage[idx].MA3RisingColor == mA3RisingColor && cacheMAPackage[idx].MA3RisingColorOpacity == mA3RisingColorOpacity && cacheMAPackage[idx].MA3Type == mA3Type && cacheMAPackage[idx].MA4FallingColor == mA4FallingColor && cacheMAPackage[idx].MA4FallingColorOpacity == mA4FallingColorOpacity && cacheMAPackage[idx].MA4LineDashStyle == mA4LineDashStyle && cacheMAPackage[idx].MA4LinePlotStyle == mA4LinePlotStyle && cacheMAPackage[idx].MA4LineWidth == mA4LineWidth && cacheMAPackage[idx].MA4Period == mA4Period && cacheMAPackage[idx].MA4PriceType == mA4PriceType && cacheMAPackage[idx].MA4RisingColor == mA4RisingColor && cacheMAPackage[idx].MA4RisingColorOpacity == mA4RisingColorOpacity && cacheMAPackage[idx].MA4Type == mA4Type && cacheMAPackage[idx].MA5FallingColor == mA5FallingColor && cacheMAPackage[idx].MA5FallingColorOpacity == mA5FallingColorOpacity && cacheMAPackage[idx].MA5LineDashStyle == mA5LineDashStyle && cacheMAPackage[idx].MA5LinePlotStyle == mA5LinePlotStyle && cacheMAPackage[idx].MA5LineWidth == mA5LineWidth && cacheMAPackage[idx].MA5Period == mA5Period && cacheMAPackage[idx].MA5PriceType == mA5PriceType && cacheMAPackage[idx].MA5RisingColor == mA5RisingColor && cacheMAPackage[idx].MA5RisingColorOpacity == mA5RisingColorOpacity && cacheMAPackage[idx].MA5Type == mA5Type && cacheMAPackage[idx].MA6FallingColor == mA6FallingColor && cacheMAPackage[idx].MA6FallingColorOpacity == mA6FallingColorOpacity && cacheMAPackage[idx].MA6LineDashStyle == mA6LineDashStyle && cacheMAPackage[idx].MA6LinePlotStyle == mA6LinePlotStyle && cacheMAPackage[idx].MA6LineWidth == mA6LineWidth && cacheMAPackage[idx].MA6Period == mA6Period && cacheMAPackage[idx].MA6PriceType == mA6PriceType && cacheMAPackage[idx].MA6RisingColor == mA6RisingColor && cacheMAPackage[idx].MA6RisingColorOpacity == mA6RisingColorOpacity && cacheMAPackage[idx].MA6Type == mA6Type && cacheMAPackage[idx].MA7FallingColor == mA7FallingColor && cacheMAPackage[idx].MA7FallingColorOpacity == mA7FallingColorOpacity && cacheMAPackage[idx].MA7LineDashStyle == mA7LineDashStyle && cacheMAPackage[idx].MA7LinePlotStyle == mA7LinePlotStyle && cacheMAPackage[idx].MA7LineWidth == mA7LineWidth && cacheMAPackage[idx].MA7Period == mA7Period && cacheMAPackage[idx].MA7PriceType == mA7PriceType && cacheMAPackage[idx].MA7RisingColor == mA7RisingColor && cacheMAPackage[idx].MA7RisingColorOpacity == mA7RisingColorOpacity && cacheMAPackage[idx].MA7Type == mA7Type && cacheMAPackage[idx].SuperSmootherFilterPoles1 == superSmootherFilterPoles1 && cacheMAPackage[idx].SuperSmootherFilterPoles2 == superSmootherFilterPoles2 && cacheMAPackage[idx].SuperSmootherFilterPoles3 == superSmootherFilterPoles3 && cacheMAPackage[idx].SuperSmootherFilterPoles4 == superSmootherFilterPoles4 && cacheMAPackage[idx].SuperSmootherFilterPoles5 == superSmootherFilterPoles5 && cacheMAPackage[idx].SuperSmootherFilterPoles6 == superSmootherFilterPoles6 && cacheMAPackage[idx].SuperSmootherFilterPoles7 == superSmootherFilterPoles7 && cacheMAPackage[idx].T3tCount1 == t3tCount1 && cacheMAPackage[idx].T3tCount2 == t3tCount2 && cacheMAPackage[idx].T3tCount3 == t3tCount3 && cacheMAPackage[idx].T3tCount4 == t3tCount4 && cacheMAPackage[idx].T3tCount5 == t3tCount5 && cacheMAPackage[idx].T3tCount6 == t3tCount6 && cacheMAPackage[idx].T3tCount7 == t3tCount7 && Math.Abs(cacheMAPackage[idx].T3vFactor1 - t3vFactor1) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor2 - t3vFactor2) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor3 - t3vFactor3) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor4 - t3vFactor4) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor5 - t3vFactor5) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor6 - t3vFactor6) <= double.Epsilon && Math.Abs(cacheMAPackage[idx].T3vFactor7 - t3vFactor7) <= double.Epsilon && cacheMAPackage[idx].UseMA1 == useMA1 && cacheMAPackage[idx].UseMA2 == useMA2 && cacheMAPackage[idx].UseMA3 == useMA3 && cacheMAPackage[idx].UseMA4 == useMA4 && cacheMAPackage[idx].UseMA5 == useMA5 && cacheMAPackage[idx].UseMA6 == useMA6 && cacheMAPackage[idx].UseMA7 == useMA7 && cacheMAPackage[idx].EqualsInput(input))
                            return cacheMAPackage[idx];

                MAPackage indicator = new MAPackage();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ButterWorthFilterPoles1 = butterWorthFilterPoles1;
                indicator.ButterWorthFilterPoles2 = butterWorthFilterPoles2;
                indicator.ButterWorthFilterPoles3 = butterWorthFilterPoles3;
                indicator.ButterWorthFilterPoles4 = butterWorthFilterPoles4;
                indicator.ButterWorthFilterPoles5 = butterWorthFilterPoles5;
                indicator.ButterWorthFilterPoles6 = butterWorthFilterPoles6;
                indicator.ButterWorthFilterPoles7 = butterWorthFilterPoles7;
                indicator.GaussianFilterPoles1 = gaussianFilterPoles1;
                indicator.GaussianFilterPoles2 = gaussianFilterPoles2;
                indicator.GaussianFilterPoles3 = gaussianFilterPoles3;
                indicator.GaussianFilterPoles4 = gaussianFilterPoles4;
                indicator.GaussianFilterPoles5 = gaussianFilterPoles5;
                indicator.GaussianFilterPoles6 = gaussianFilterPoles6;
                indicator.GaussianFilterPoles7 = gaussianFilterPoles7;
                indicator.JMAPhase1 = jMAPhase1;
                indicator.JMAPhase2 = jMAPhase2;
                indicator.JMAPhase3 = jMAPhase3;
                indicator.JMAPhase4 = jMAPhase4;
                indicator.JMAPhase5 = jMAPhase5;
                indicator.JMAPhase6 = jMAPhase6;
                indicator.JMAPhase7 = jMAPhase7;
                indicator.MA1FallingColor = mA1FallingColor;
                indicator.MA1FallingColorOpacity = mA1FallingColorOpacity;
                indicator.MA1LineDashStyle = mA1LineDashStyle;
                indicator.MA1LinePlotStyle = mA1LinePlotStyle;
                indicator.MA1LineWidth = mA1LineWidth;
                indicator.MA1Period = mA1Period;
                indicator.MA1PriceType = mA1PriceType;
                indicator.MA1RisingColor = mA1RisingColor;
                indicator.MA1RisingColorOpacity = mA1RisingColorOpacity;
                indicator.MA1Type = mA1Type;
                indicator.MA2FallingColor = mA2FallingColor;
                indicator.MA2FallingColorOpacity = mA2FallingColorOpacity;
                indicator.MA2LineDashStyle = mA2LineDashStyle;
                indicator.MA2LinePlotStyle = mA2LinePlotStyle;
                indicator.MA2LineWidth = mA2LineWidth;
                indicator.MA2Period = mA2Period;
                indicator.MA2PriceType = mA2PriceType;
                indicator.MA2RisingColor = mA2RisingColor;
                indicator.MA2RisingColorOpacity = mA2RisingColorOpacity;
                indicator.MA2Type = mA2Type;
                indicator.MA3FallingColor = mA3FallingColor;
                indicator.MA3FallingColorOpacity = mA3FallingColorOpacity;
                indicator.MA3LineDashStyle = mA3LineDashStyle;
                indicator.MA3LinePlotStyle = mA3LinePlotStyle;
                indicator.MA3LineWidth = mA3LineWidth;
                indicator.MA3Period = mA3Period;
                indicator.MA3PriceType = mA3PriceType;
                indicator.MA3RisingColor = mA3RisingColor;
                indicator.MA3RisingColorOpacity = mA3RisingColorOpacity;
                indicator.MA3Type = mA3Type;
                indicator.MA4FallingColor = mA4FallingColor;
                indicator.MA4FallingColorOpacity = mA4FallingColorOpacity;
                indicator.MA4LineDashStyle = mA4LineDashStyle;
                indicator.MA4LinePlotStyle = mA4LinePlotStyle;
                indicator.MA4LineWidth = mA4LineWidth;
                indicator.MA4Period = mA4Period;
                indicator.MA4PriceType = mA4PriceType;
                indicator.MA4RisingColor = mA4RisingColor;
                indicator.MA4RisingColorOpacity = mA4RisingColorOpacity;
                indicator.MA4Type = mA4Type;
                indicator.MA5FallingColor = mA5FallingColor;
                indicator.MA5FallingColorOpacity = mA5FallingColorOpacity;
                indicator.MA5LineDashStyle = mA5LineDashStyle;
                indicator.MA5LinePlotStyle = mA5LinePlotStyle;
                indicator.MA5LineWidth = mA5LineWidth;
                indicator.MA5Period = mA5Period;
                indicator.MA5PriceType = mA5PriceType;
                indicator.MA5RisingColor = mA5RisingColor;
                indicator.MA5RisingColorOpacity = mA5RisingColorOpacity;
                indicator.MA5Type = mA5Type;
                indicator.MA6FallingColor = mA6FallingColor;
                indicator.MA6FallingColorOpacity = mA6FallingColorOpacity;
                indicator.MA6LineDashStyle = mA6LineDashStyle;
                indicator.MA6LinePlotStyle = mA6LinePlotStyle;
                indicator.MA6LineWidth = mA6LineWidth;
                indicator.MA6Period = mA6Period;
                indicator.MA6PriceType = mA6PriceType;
                indicator.MA6RisingColor = mA6RisingColor;
                indicator.MA6RisingColorOpacity = mA6RisingColorOpacity;
                indicator.MA6Type = mA6Type;
                indicator.MA7FallingColor = mA7FallingColor;
                indicator.MA7FallingColorOpacity = mA7FallingColorOpacity;
                indicator.MA7LineDashStyle = mA7LineDashStyle;
                indicator.MA7LinePlotStyle = mA7LinePlotStyle;
                indicator.MA7LineWidth = mA7LineWidth;
                indicator.MA7Period = mA7Period;
                indicator.MA7PriceType = mA7PriceType;
                indicator.MA7RisingColor = mA7RisingColor;
                indicator.MA7RisingColorOpacity = mA7RisingColorOpacity;
                indicator.MA7Type = mA7Type;
                indicator.SuperSmootherFilterPoles1 = superSmootherFilterPoles1;
                indicator.SuperSmootherFilterPoles2 = superSmootherFilterPoles2;
                indicator.SuperSmootherFilterPoles3 = superSmootherFilterPoles3;
                indicator.SuperSmootherFilterPoles4 = superSmootherFilterPoles4;
                indicator.SuperSmootherFilterPoles5 = superSmootherFilterPoles5;
                indicator.SuperSmootherFilterPoles6 = superSmootherFilterPoles6;
                indicator.SuperSmootherFilterPoles7 = superSmootherFilterPoles7;
                indicator.T3tCount1 = t3tCount1;
                indicator.T3tCount2 = t3tCount2;
                indicator.T3tCount3 = t3tCount3;
                indicator.T3tCount4 = t3tCount4;
                indicator.T3tCount5 = t3tCount5;
                indicator.T3tCount6 = t3tCount6;
                indicator.T3tCount7 = t3tCount7;
                indicator.T3vFactor1 = t3vFactor1;
                indicator.T3vFactor2 = t3vFactor2;
                indicator.T3vFactor3 = t3vFactor3;
                indicator.T3vFactor4 = t3vFactor4;
                indicator.T3vFactor5 = t3vFactor5;
                indicator.T3vFactor6 = t3vFactor6;
                indicator.T3vFactor7 = t3vFactor7;
                indicator.UseMA1 = useMA1;
                indicator.UseMA2 = useMA2;
                indicator.UseMA3 = useMA3;
                indicator.UseMA4 = useMA4;
                indicator.UseMA5 = useMA5;
                indicator.UseMA6 = useMA6;
                indicator.UseMA7 = useMA7;
                Indicators.Add(indicator);
                indicator.SetUp();

                MAPackage[] tmp = new MAPackage[cacheMAPackage == null ? 1 : cacheMAPackage.Length + 1];
                if (cacheMAPackage != null)
                    cacheMAPackage.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheMAPackage = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// MAPackage V1.3
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MAPackage MAPackage(int butterWorthFilterPoles1, int butterWorthFilterPoles2, int butterWorthFilterPoles3, int butterWorthFilterPoles4, int butterWorthFilterPoles5, int butterWorthFilterPoles6, int butterWorthFilterPoles7, int gaussianFilterPoles1, int gaussianFilterPoles2, int gaussianFilterPoles3, int gaussianFilterPoles4, int gaussianFilterPoles5, int gaussianFilterPoles6, int gaussianFilterPoles7, int jMAPhase1, int jMAPhase2, int jMAPhase3, int jMAPhase4, int jMAPhase5, int jMAPhase6, int jMAPhase7, Color mA1FallingColor, int mA1FallingColorOpacity, MAPackageDashStyleV13 mA1LineDashStyle, MAPackagePlotStyleV13 mA1LinePlotStyle, int mA1LineWidth, int mA1Period, MAPackageMAPriceTypesV13 mA1PriceType, Color mA1RisingColor, int mA1RisingColorOpacity, MAPackageMATypeV13 mA1Type, Color mA2FallingColor, int mA2FallingColorOpacity, MAPackageDashStyleV13 mA2LineDashStyle, MAPackagePlotStyleV13 mA2LinePlotStyle, int mA2LineWidth, int mA2Period, MAPackageMAPriceTypesV13 mA2PriceType, Color mA2RisingColor, int mA2RisingColorOpacity, MAPackageMATypeV13 mA2Type, Color mA3FallingColor, int mA3FallingColorOpacity, MAPackageDashStyleV13 mA3LineDashStyle, MAPackagePlotStyleV13 mA3LinePlotStyle, int mA3LineWidth, int mA3Period, MAPackageMAPriceTypesV13 mA3PriceType, Color mA3RisingColor, int mA3RisingColorOpacity, MAPackageMATypeV13 mA3Type, Color mA4FallingColor, int mA4FallingColorOpacity, MAPackageDashStyleV13 mA4LineDashStyle, MAPackagePlotStyleV13 mA4LinePlotStyle, int mA4LineWidth, int mA4Period, MAPackageMAPriceTypesV13 mA4PriceType, Color mA4RisingColor, int mA4RisingColorOpacity, MAPackageMATypeV13 mA4Type, Color mA5FallingColor, int mA5FallingColorOpacity, MAPackageDashStyleV13 mA5LineDashStyle, MAPackagePlotStyleV13 mA5LinePlotStyle, int mA5LineWidth, int mA5Period, MAPackageMAPriceTypesV13 mA5PriceType, Color mA5RisingColor, int mA5RisingColorOpacity, MAPackageMATypeV13 mA5Type, Color mA6FallingColor, int mA6FallingColorOpacity, MAPackageDashStyleV13 mA6LineDashStyle, MAPackagePlotStyleV13 mA6LinePlotStyle, int mA6LineWidth, int mA6Period, MAPackageMAPriceTypesV13 mA6PriceType, Color mA6RisingColor, int mA6RisingColorOpacity, MAPackageMATypeV13 mA6Type, Color mA7FallingColor, int mA7FallingColorOpacity, MAPackageDashStyleV13 mA7LineDashStyle, MAPackagePlotStyleV13 mA7LinePlotStyle, int mA7LineWidth, int mA7Period, MAPackageMAPriceTypesV13 mA7PriceType, Color mA7RisingColor, int mA7RisingColorOpacity, MAPackageMATypeV13 mA7Type, int superSmootherFilterPoles1, int superSmootherFilterPoles2, int superSmootherFilterPoles3, int superSmootherFilterPoles4, int superSmootherFilterPoles5, int superSmootherFilterPoles6, int superSmootherFilterPoles7, int t3tCount1, int t3tCount2, int t3tCount3, int t3tCount4, int t3tCount5, int t3tCount6, int t3tCount7, double t3vFactor1, double t3vFactor2, double t3vFactor3, double t3vFactor4, double t3vFactor5, double t3vFactor6, double t3vFactor7, bool useMA1, bool useMA2, bool useMA3, bool useMA4, bool useMA5, bool useMA6, bool useMA7)
        {
            return _indicator.MAPackage(Input, butterWorthFilterPoles1, butterWorthFilterPoles2, butterWorthFilterPoles3, butterWorthFilterPoles4, butterWorthFilterPoles5, butterWorthFilterPoles6, butterWorthFilterPoles7, gaussianFilterPoles1, gaussianFilterPoles2, gaussianFilterPoles3, gaussianFilterPoles4, gaussianFilterPoles5, gaussianFilterPoles6, gaussianFilterPoles7, jMAPhase1, jMAPhase2, jMAPhase3, jMAPhase4, jMAPhase5, jMAPhase6, jMAPhase7, mA1FallingColor, mA1FallingColorOpacity, mA1LineDashStyle, mA1LinePlotStyle, mA1LineWidth, mA1Period, mA1PriceType, mA1RisingColor, mA1RisingColorOpacity, mA1Type, mA2FallingColor, mA2FallingColorOpacity, mA2LineDashStyle, mA2LinePlotStyle, mA2LineWidth, mA2Period, mA2PriceType, mA2RisingColor, mA2RisingColorOpacity, mA2Type, mA3FallingColor, mA3FallingColorOpacity, mA3LineDashStyle, mA3LinePlotStyle, mA3LineWidth, mA3Period, mA3PriceType, mA3RisingColor, mA3RisingColorOpacity, mA3Type, mA4FallingColor, mA4FallingColorOpacity, mA4LineDashStyle, mA4LinePlotStyle, mA4LineWidth, mA4Period, mA4PriceType, mA4RisingColor, mA4RisingColorOpacity, mA4Type, mA5FallingColor, mA5FallingColorOpacity, mA5LineDashStyle, mA5LinePlotStyle, mA5LineWidth, mA5Period, mA5PriceType, mA5RisingColor, mA5RisingColorOpacity, mA5Type, mA6FallingColor, mA6FallingColorOpacity, mA6LineDashStyle, mA6LinePlotStyle, mA6LineWidth, mA6Period, mA6PriceType, mA6RisingColor, mA6RisingColorOpacity, mA6Type, mA7FallingColor, mA7FallingColorOpacity, mA7LineDashStyle, mA7LinePlotStyle, mA7LineWidth, mA7Period, mA7PriceType, mA7RisingColor, mA7RisingColorOpacity, mA7Type, superSmootherFilterPoles1, superSmootherFilterPoles2, superSmootherFilterPoles3, superSmootherFilterPoles4, superSmootherFilterPoles5, superSmootherFilterPoles6, superSmootherFilterPoles7, t3tCount1, t3tCount2, t3tCount3, t3tCount4, t3tCount5, t3tCount6, t3tCount7, t3vFactor1, t3vFactor2, t3vFactor3, t3vFactor4, t3vFactor5, t3vFactor6, t3vFactor7, useMA1, useMA2, useMA3, useMA4, useMA5, useMA6, useMA7);
        }

        /// <summary>
        /// MAPackage V1.3
        /// </summary>
        /// <returns></returns>
        public Indicator.MAPackage MAPackage(Data.IDataSeries input, int butterWorthFilterPoles1, int butterWorthFilterPoles2, int butterWorthFilterPoles3, int butterWorthFilterPoles4, int butterWorthFilterPoles5, int butterWorthFilterPoles6, int butterWorthFilterPoles7, int gaussianFilterPoles1, int gaussianFilterPoles2, int gaussianFilterPoles3, int gaussianFilterPoles4, int gaussianFilterPoles5, int gaussianFilterPoles6, int gaussianFilterPoles7, int jMAPhase1, int jMAPhase2, int jMAPhase3, int jMAPhase4, int jMAPhase5, int jMAPhase6, int jMAPhase7, Color mA1FallingColor, int mA1FallingColorOpacity, MAPackageDashStyleV13 mA1LineDashStyle, MAPackagePlotStyleV13 mA1LinePlotStyle, int mA1LineWidth, int mA1Period, MAPackageMAPriceTypesV13 mA1PriceType, Color mA1RisingColor, int mA1RisingColorOpacity, MAPackageMATypeV13 mA1Type, Color mA2FallingColor, int mA2FallingColorOpacity, MAPackageDashStyleV13 mA2LineDashStyle, MAPackagePlotStyleV13 mA2LinePlotStyle, int mA2LineWidth, int mA2Period, MAPackageMAPriceTypesV13 mA2PriceType, Color mA2RisingColor, int mA2RisingColorOpacity, MAPackageMATypeV13 mA2Type, Color mA3FallingColor, int mA3FallingColorOpacity, MAPackageDashStyleV13 mA3LineDashStyle, MAPackagePlotStyleV13 mA3LinePlotStyle, int mA3LineWidth, int mA3Period, MAPackageMAPriceTypesV13 mA3PriceType, Color mA3RisingColor, int mA3RisingColorOpacity, MAPackageMATypeV13 mA3Type, Color mA4FallingColor, int mA4FallingColorOpacity, MAPackageDashStyleV13 mA4LineDashStyle, MAPackagePlotStyleV13 mA4LinePlotStyle, int mA4LineWidth, int mA4Period, MAPackageMAPriceTypesV13 mA4PriceType, Color mA4RisingColor, int mA4RisingColorOpacity, MAPackageMATypeV13 mA4Type, Color mA5FallingColor, int mA5FallingColorOpacity, MAPackageDashStyleV13 mA5LineDashStyle, MAPackagePlotStyleV13 mA5LinePlotStyle, int mA5LineWidth, int mA5Period, MAPackageMAPriceTypesV13 mA5PriceType, Color mA5RisingColor, int mA5RisingColorOpacity, MAPackageMATypeV13 mA5Type, Color mA6FallingColor, int mA6FallingColorOpacity, MAPackageDashStyleV13 mA6LineDashStyle, MAPackagePlotStyleV13 mA6LinePlotStyle, int mA6LineWidth, int mA6Period, MAPackageMAPriceTypesV13 mA6PriceType, Color mA6RisingColor, int mA6RisingColorOpacity, MAPackageMATypeV13 mA6Type, Color mA7FallingColor, int mA7FallingColorOpacity, MAPackageDashStyleV13 mA7LineDashStyle, MAPackagePlotStyleV13 mA7LinePlotStyle, int mA7LineWidth, int mA7Period, MAPackageMAPriceTypesV13 mA7PriceType, Color mA7RisingColor, int mA7RisingColorOpacity, MAPackageMATypeV13 mA7Type, int superSmootherFilterPoles1, int superSmootherFilterPoles2, int superSmootherFilterPoles3, int superSmootherFilterPoles4, int superSmootherFilterPoles5, int superSmootherFilterPoles6, int superSmootherFilterPoles7, int t3tCount1, int t3tCount2, int t3tCount3, int t3tCount4, int t3tCount5, int t3tCount6, int t3tCount7, double t3vFactor1, double t3vFactor2, double t3vFactor3, double t3vFactor4, double t3vFactor5, double t3vFactor6, double t3vFactor7, bool useMA1, bool useMA2, bool useMA3, bool useMA4, bool useMA5, bool useMA6, bool useMA7)
        {
            return _indicator.MAPackage(input, butterWorthFilterPoles1, butterWorthFilterPoles2, butterWorthFilterPoles3, butterWorthFilterPoles4, butterWorthFilterPoles5, butterWorthFilterPoles6, butterWorthFilterPoles7, gaussianFilterPoles1, gaussianFilterPoles2, gaussianFilterPoles3, gaussianFilterPoles4, gaussianFilterPoles5, gaussianFilterPoles6, gaussianFilterPoles7, jMAPhase1, jMAPhase2, jMAPhase3, jMAPhase4, jMAPhase5, jMAPhase6, jMAPhase7, mA1FallingColor, mA1FallingColorOpacity, mA1LineDashStyle, mA1LinePlotStyle, mA1LineWidth, mA1Period, mA1PriceType, mA1RisingColor, mA1RisingColorOpacity, mA1Type, mA2FallingColor, mA2FallingColorOpacity, mA2LineDashStyle, mA2LinePlotStyle, mA2LineWidth, mA2Period, mA2PriceType, mA2RisingColor, mA2RisingColorOpacity, mA2Type, mA3FallingColor, mA3FallingColorOpacity, mA3LineDashStyle, mA3LinePlotStyle, mA3LineWidth, mA3Period, mA3PriceType, mA3RisingColor, mA3RisingColorOpacity, mA3Type, mA4FallingColor, mA4FallingColorOpacity, mA4LineDashStyle, mA4LinePlotStyle, mA4LineWidth, mA4Period, mA4PriceType, mA4RisingColor, mA4RisingColorOpacity, mA4Type, mA5FallingColor, mA5FallingColorOpacity, mA5LineDashStyle, mA5LinePlotStyle, mA5LineWidth, mA5Period, mA5PriceType, mA5RisingColor, mA5RisingColorOpacity, mA5Type, mA6FallingColor, mA6FallingColorOpacity, mA6LineDashStyle, mA6LinePlotStyle, mA6LineWidth, mA6Period, mA6PriceType, mA6RisingColor, mA6RisingColorOpacity, mA6Type, mA7FallingColor, mA7FallingColorOpacity, mA7LineDashStyle, mA7LinePlotStyle, mA7LineWidth, mA7Period, mA7PriceType, mA7RisingColor, mA7RisingColorOpacity, mA7Type, superSmootherFilterPoles1, superSmootherFilterPoles2, superSmootherFilterPoles3, superSmootherFilterPoles4, superSmootherFilterPoles5, superSmootherFilterPoles6, superSmootherFilterPoles7, t3tCount1, t3tCount2, t3tCount3, t3tCount4, t3tCount5, t3tCount6, t3tCount7, t3vFactor1, t3vFactor2, t3vFactor3, t3vFactor4, t3vFactor5, t3vFactor6, t3vFactor7, useMA1, useMA2, useMA3, useMA4, useMA5, useMA6, useMA7);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// MAPackage V1.3
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MAPackage MAPackage(int butterWorthFilterPoles1, int butterWorthFilterPoles2, int butterWorthFilterPoles3, int butterWorthFilterPoles4, int butterWorthFilterPoles5, int butterWorthFilterPoles6, int butterWorthFilterPoles7, int gaussianFilterPoles1, int gaussianFilterPoles2, int gaussianFilterPoles3, int gaussianFilterPoles4, int gaussianFilterPoles5, int gaussianFilterPoles6, int gaussianFilterPoles7, int jMAPhase1, int jMAPhase2, int jMAPhase3, int jMAPhase4, int jMAPhase5, int jMAPhase6, int jMAPhase7, Color mA1FallingColor, int mA1FallingColorOpacity, MAPackageDashStyleV13 mA1LineDashStyle, MAPackagePlotStyleV13 mA1LinePlotStyle, int mA1LineWidth, int mA1Period, MAPackageMAPriceTypesV13 mA1PriceType, Color mA1RisingColor, int mA1RisingColorOpacity, MAPackageMATypeV13 mA1Type, Color mA2FallingColor, int mA2FallingColorOpacity, MAPackageDashStyleV13 mA2LineDashStyle, MAPackagePlotStyleV13 mA2LinePlotStyle, int mA2LineWidth, int mA2Period, MAPackageMAPriceTypesV13 mA2PriceType, Color mA2RisingColor, int mA2RisingColorOpacity, MAPackageMATypeV13 mA2Type, Color mA3FallingColor, int mA3FallingColorOpacity, MAPackageDashStyleV13 mA3LineDashStyle, MAPackagePlotStyleV13 mA3LinePlotStyle, int mA3LineWidth, int mA3Period, MAPackageMAPriceTypesV13 mA3PriceType, Color mA3RisingColor, int mA3RisingColorOpacity, MAPackageMATypeV13 mA3Type, Color mA4FallingColor, int mA4FallingColorOpacity, MAPackageDashStyleV13 mA4LineDashStyle, MAPackagePlotStyleV13 mA4LinePlotStyle, int mA4LineWidth, int mA4Period, MAPackageMAPriceTypesV13 mA4PriceType, Color mA4RisingColor, int mA4RisingColorOpacity, MAPackageMATypeV13 mA4Type, Color mA5FallingColor, int mA5FallingColorOpacity, MAPackageDashStyleV13 mA5LineDashStyle, MAPackagePlotStyleV13 mA5LinePlotStyle, int mA5LineWidth, int mA5Period, MAPackageMAPriceTypesV13 mA5PriceType, Color mA5RisingColor, int mA5RisingColorOpacity, MAPackageMATypeV13 mA5Type, Color mA6FallingColor, int mA6FallingColorOpacity, MAPackageDashStyleV13 mA6LineDashStyle, MAPackagePlotStyleV13 mA6LinePlotStyle, int mA6LineWidth, int mA6Period, MAPackageMAPriceTypesV13 mA6PriceType, Color mA6RisingColor, int mA6RisingColorOpacity, MAPackageMATypeV13 mA6Type, Color mA7FallingColor, int mA7FallingColorOpacity, MAPackageDashStyleV13 mA7LineDashStyle, MAPackagePlotStyleV13 mA7LinePlotStyle, int mA7LineWidth, int mA7Period, MAPackageMAPriceTypesV13 mA7PriceType, Color mA7RisingColor, int mA7RisingColorOpacity, MAPackageMATypeV13 mA7Type, int superSmootherFilterPoles1, int superSmootherFilterPoles2, int superSmootherFilterPoles3, int superSmootherFilterPoles4, int superSmootherFilterPoles5, int superSmootherFilterPoles6, int superSmootherFilterPoles7, int t3tCount1, int t3tCount2, int t3tCount3, int t3tCount4, int t3tCount5, int t3tCount6, int t3tCount7, double t3vFactor1, double t3vFactor2, double t3vFactor3, double t3vFactor4, double t3vFactor5, double t3vFactor6, double t3vFactor7, bool useMA1, bool useMA2, bool useMA3, bool useMA4, bool useMA5, bool useMA6, bool useMA7)
        {
            return _indicator.MAPackage(Input, butterWorthFilterPoles1, butterWorthFilterPoles2, butterWorthFilterPoles3, butterWorthFilterPoles4, butterWorthFilterPoles5, butterWorthFilterPoles6, butterWorthFilterPoles7, gaussianFilterPoles1, gaussianFilterPoles2, gaussianFilterPoles3, gaussianFilterPoles4, gaussianFilterPoles5, gaussianFilterPoles6, gaussianFilterPoles7, jMAPhase1, jMAPhase2, jMAPhase3, jMAPhase4, jMAPhase5, jMAPhase6, jMAPhase7, mA1FallingColor, mA1FallingColorOpacity, mA1LineDashStyle, mA1LinePlotStyle, mA1LineWidth, mA1Period, mA1PriceType, mA1RisingColor, mA1RisingColorOpacity, mA1Type, mA2FallingColor, mA2FallingColorOpacity, mA2LineDashStyle, mA2LinePlotStyle, mA2LineWidth, mA2Period, mA2PriceType, mA2RisingColor, mA2RisingColorOpacity, mA2Type, mA3FallingColor, mA3FallingColorOpacity, mA3LineDashStyle, mA3LinePlotStyle, mA3LineWidth, mA3Period, mA3PriceType, mA3RisingColor, mA3RisingColorOpacity, mA3Type, mA4FallingColor, mA4FallingColorOpacity, mA4LineDashStyle, mA4LinePlotStyle, mA4LineWidth, mA4Period, mA4PriceType, mA4RisingColor, mA4RisingColorOpacity, mA4Type, mA5FallingColor, mA5FallingColorOpacity, mA5LineDashStyle, mA5LinePlotStyle, mA5LineWidth, mA5Period, mA5PriceType, mA5RisingColor, mA5RisingColorOpacity, mA5Type, mA6FallingColor, mA6FallingColorOpacity, mA6LineDashStyle, mA6LinePlotStyle, mA6LineWidth, mA6Period, mA6PriceType, mA6RisingColor, mA6RisingColorOpacity, mA6Type, mA7FallingColor, mA7FallingColorOpacity, mA7LineDashStyle, mA7LinePlotStyle, mA7LineWidth, mA7Period, mA7PriceType, mA7RisingColor, mA7RisingColorOpacity, mA7Type, superSmootherFilterPoles1, superSmootherFilterPoles2, superSmootherFilterPoles3, superSmootherFilterPoles4, superSmootherFilterPoles5, superSmootherFilterPoles6, superSmootherFilterPoles7, t3tCount1, t3tCount2, t3tCount3, t3tCount4, t3tCount5, t3tCount6, t3tCount7, t3vFactor1, t3vFactor2, t3vFactor3, t3vFactor4, t3vFactor5, t3vFactor6, t3vFactor7, useMA1, useMA2, useMA3, useMA4, useMA5, useMA6, useMA7);
        }

        /// <summary>
        /// MAPackage V1.3
        /// </summary>
        /// <returns></returns>
        public Indicator.MAPackage MAPackage(Data.IDataSeries input, int butterWorthFilterPoles1, int butterWorthFilterPoles2, int butterWorthFilterPoles3, int butterWorthFilterPoles4, int butterWorthFilterPoles5, int butterWorthFilterPoles6, int butterWorthFilterPoles7, int gaussianFilterPoles1, int gaussianFilterPoles2, int gaussianFilterPoles3, int gaussianFilterPoles4, int gaussianFilterPoles5, int gaussianFilterPoles6, int gaussianFilterPoles7, int jMAPhase1, int jMAPhase2, int jMAPhase3, int jMAPhase4, int jMAPhase5, int jMAPhase6, int jMAPhase7, Color mA1FallingColor, int mA1FallingColorOpacity, MAPackageDashStyleV13 mA1LineDashStyle, MAPackagePlotStyleV13 mA1LinePlotStyle, int mA1LineWidth, int mA1Period, MAPackageMAPriceTypesV13 mA1PriceType, Color mA1RisingColor, int mA1RisingColorOpacity, MAPackageMATypeV13 mA1Type, Color mA2FallingColor, int mA2FallingColorOpacity, MAPackageDashStyleV13 mA2LineDashStyle, MAPackagePlotStyleV13 mA2LinePlotStyle, int mA2LineWidth, int mA2Period, MAPackageMAPriceTypesV13 mA2PriceType, Color mA2RisingColor, int mA2RisingColorOpacity, MAPackageMATypeV13 mA2Type, Color mA3FallingColor, int mA3FallingColorOpacity, MAPackageDashStyleV13 mA3LineDashStyle, MAPackagePlotStyleV13 mA3LinePlotStyle, int mA3LineWidth, int mA3Period, MAPackageMAPriceTypesV13 mA3PriceType, Color mA3RisingColor, int mA3RisingColorOpacity, MAPackageMATypeV13 mA3Type, Color mA4FallingColor, int mA4FallingColorOpacity, MAPackageDashStyleV13 mA4LineDashStyle, MAPackagePlotStyleV13 mA4LinePlotStyle, int mA4LineWidth, int mA4Period, MAPackageMAPriceTypesV13 mA4PriceType, Color mA4RisingColor, int mA4RisingColorOpacity, MAPackageMATypeV13 mA4Type, Color mA5FallingColor, int mA5FallingColorOpacity, MAPackageDashStyleV13 mA5LineDashStyle, MAPackagePlotStyleV13 mA5LinePlotStyle, int mA5LineWidth, int mA5Period, MAPackageMAPriceTypesV13 mA5PriceType, Color mA5RisingColor, int mA5RisingColorOpacity, MAPackageMATypeV13 mA5Type, Color mA6FallingColor, int mA6FallingColorOpacity, MAPackageDashStyleV13 mA6LineDashStyle, MAPackagePlotStyleV13 mA6LinePlotStyle, int mA6LineWidth, int mA6Period, MAPackageMAPriceTypesV13 mA6PriceType, Color mA6RisingColor, int mA6RisingColorOpacity, MAPackageMATypeV13 mA6Type, Color mA7FallingColor, int mA7FallingColorOpacity, MAPackageDashStyleV13 mA7LineDashStyle, MAPackagePlotStyleV13 mA7LinePlotStyle, int mA7LineWidth, int mA7Period, MAPackageMAPriceTypesV13 mA7PriceType, Color mA7RisingColor, int mA7RisingColorOpacity, MAPackageMATypeV13 mA7Type, int superSmootherFilterPoles1, int superSmootherFilterPoles2, int superSmootherFilterPoles3, int superSmootherFilterPoles4, int superSmootherFilterPoles5, int superSmootherFilterPoles6, int superSmootherFilterPoles7, int t3tCount1, int t3tCount2, int t3tCount3, int t3tCount4, int t3tCount5, int t3tCount6, int t3tCount7, double t3vFactor1, double t3vFactor2, double t3vFactor3, double t3vFactor4, double t3vFactor5, double t3vFactor6, double t3vFactor7, bool useMA1, bool useMA2, bool useMA3, bool useMA4, bool useMA5, bool useMA6, bool useMA7)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MAPackage(input, butterWorthFilterPoles1, butterWorthFilterPoles2, butterWorthFilterPoles3, butterWorthFilterPoles4, butterWorthFilterPoles5, butterWorthFilterPoles6, butterWorthFilterPoles7, gaussianFilterPoles1, gaussianFilterPoles2, gaussianFilterPoles3, gaussianFilterPoles4, gaussianFilterPoles5, gaussianFilterPoles6, gaussianFilterPoles7, jMAPhase1, jMAPhase2, jMAPhase3, jMAPhase4, jMAPhase5, jMAPhase6, jMAPhase7, mA1FallingColor, mA1FallingColorOpacity, mA1LineDashStyle, mA1LinePlotStyle, mA1LineWidth, mA1Period, mA1PriceType, mA1RisingColor, mA1RisingColorOpacity, mA1Type, mA2FallingColor, mA2FallingColorOpacity, mA2LineDashStyle, mA2LinePlotStyle, mA2LineWidth, mA2Period, mA2PriceType, mA2RisingColor, mA2RisingColorOpacity, mA2Type, mA3FallingColor, mA3FallingColorOpacity, mA3LineDashStyle, mA3LinePlotStyle, mA3LineWidth, mA3Period, mA3PriceType, mA3RisingColor, mA3RisingColorOpacity, mA3Type, mA4FallingColor, mA4FallingColorOpacity, mA4LineDashStyle, mA4LinePlotStyle, mA4LineWidth, mA4Period, mA4PriceType, mA4RisingColor, mA4RisingColorOpacity, mA4Type, mA5FallingColor, mA5FallingColorOpacity, mA5LineDashStyle, mA5LinePlotStyle, mA5LineWidth, mA5Period, mA5PriceType, mA5RisingColor, mA5RisingColorOpacity, mA5Type, mA6FallingColor, mA6FallingColorOpacity, mA6LineDashStyle, mA6LinePlotStyle, mA6LineWidth, mA6Period, mA6PriceType, mA6RisingColor, mA6RisingColorOpacity, mA6Type, mA7FallingColor, mA7FallingColorOpacity, mA7LineDashStyle, mA7LinePlotStyle, mA7LineWidth, mA7Period, mA7PriceType, mA7RisingColor, mA7RisingColorOpacity, mA7Type, superSmootherFilterPoles1, superSmootherFilterPoles2, superSmootherFilterPoles3, superSmootherFilterPoles4, superSmootherFilterPoles5, superSmootherFilterPoles6, superSmootherFilterPoles7, t3tCount1, t3tCount2, t3tCount3, t3tCount4, t3tCount5, t3tCount6, t3tCount7, t3vFactor1, t3vFactor2, t3vFactor3, t3vFactor4, t3vFactor5, t3vFactor6, t3vFactor7, useMA1, useMA2, useMA3, useMA4, useMA5, useMA6, useMA7);
        }
    }
}
#endregion
